;;	Copyright © CrackTime Studios - 2020
;;
;;	This file is part of Poisoned Escape
;;
;;    Poisoned Escape is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Poisoned Escape is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Poisoned Escape.  If not, see <https://www.gnu.org/licenses/>.

.include "animations.h.s"
.include "assets/assets.h.s"
.include "entity.h.s"
.include "cpctelera.h.s"
.include "entity_templates.h.s"

;; UDRL
;; 0000 - Stop Right
;; 0001 - left
;; 0010 - right
;; 0011 - Stop Left
;; 0100 - down
;; 0101 - down-left
;; 0110 - down-right
;; 0111 - down-looking-left-----
;; 1000 - up
;; 1001 - up-left
;; 1010 - up-right
;; 1011 - up-looking-left-------
;; 1100 - Death Animation
;; 1101 - Wall Left
;; 1110 - Wall Right
;; ----

;; 0000 -> player table
_animations_player_array:
    .dw _player_stop_right
    .dw _player_left_anim
    .dw _player_right_anim
    .dw _player_stop_left
    .dw _player_down_anim
    .dw _player_down_left_anim
    .dw _player_down_right_anim
    .dw _player_downLeft_anim
    .dw _player_up_anim
    .dw _player_up_left_anim
    .dw _player_up_right_anim
    .dw _player_upLeft_anim
    .dw _player_death_anim
    .dw _player_wallLeft_anim
    .dw _player_wallRight_anim

;; 0001 -> enemies table
_animations_enemies_array:
    .dw _babosa_right_anim ;; CUando esta parado
    .dw _babosa_left_anim
    .dw _babosa_right_anim
    .dw nullptr
    .dw _spider_down_anim
    .dw nullptr
    .dw nullptr
    .dw nullptr
    .dw _spider_up_anim

_ghost_enemies_array:
    .dw _ghost_right_anim ;; CUando esta parado
    .dw _ghost_left_anim
    .dw _ghost_right_anim
    .dw _ghost_left_anim
    .dw nullptr
    .dw _ghost_left_anim
    .dw _ghost_right_anim
    .dw _ghost_left_anim
    .dw nullptr
    .dw _ghost_left_anim
    .dw _ghost_right_anim
    .dw _ghost_left_anim
    .dw nullptr
    .dw nullptr
    .dw nullptr

_bat_enemies_array:
    .dw _bat_up_anim ;; CUando esta parado
    .dw _bat_down_anim
    .dw _bat_up_anim
    .dw _bat_down_anim
    .dw nullptr
    .dw _bat_down_anim
    .dw _bat_up_anim
    .dw _bat_down_anim
    .dw nullptr
    .dw _bat_down_anim
    .dw _bat_up_anim
    .dw _bat_down_anim
    .dw nullptr
    .dw nullptr
    .dw nullptr
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Calcula el estado de la animcacion
;; ENTRADAS
;;  B - Animation status
;;  C - Animation table
;; SALIDAS
;;  HL - Pointer to animation (B)
;; MODIFICA
;;  AF
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_animations_getAnimHL::
    call calculateAnimTable
    ld a, b
    add a                 ;; Obtener posicion en array
    add_hl_a              ;; macro hace lo mismo que lo comentado

    ld a, (hl)
    inc hl
    ld h, (hl)            ;; Aquello a lo que apunta HL lo cargo en HL -> HL = (*HL)
    ld l, a

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Obtiene el array correspondiente a la entidad
;; ENTRADAS
;;  B - Animation status
;;  C - Animation table
;; SALIDAS
;;  HL - Pointer to array table (B)
;; MODIFICA
;;  AF
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
calculateAnimTable::
    ld a, #0                ;; Player animation table
    cp c
    jr z, _playertable      ;; si es 0

    ld a, #1                ;; Enemies animation table
    cp c
    jr z, _enemiestable      ;; si es 0

    ld a, #2                ;; Enemies animation table
    cp c
    jr z, _ghosttable      ;; si es 0

    ld a, #3                ;; Enemies animation table
    cp c
    jr z, _battable      ;; si es 0

_playertable:
    ld hl, #_animations_player_array
    jr _end

_enemiestable:
    ld hl, #_animations_enemies_array
    jr _end
_ghosttable:
    ld hl, #_ghost_enemies_array
    jr _end

_battable:
    ld hl, #_bat_enemies_array
    jr _end


_end:
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;      Player animations
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
_player_right_anim::
    .db player_frames
    .dw _sp_athe_03
    .db player_frames
    .dw _sp_athe_04
    .db player_frames
    .dw _sp_athe_05
    .db player_frames
    .dw _sp_athe_04
    .dw nullptr
    .dw _player_right_anim

_player_left_anim::
    .db player_frames
    .dw _sp_athe_09
    .db player_frames
    .dw _sp_athe_10
    .db player_frames
    .dw _sp_athe_11
    .db player_frames
    .dw _sp_athe_10
    .dw nullptr
    .dw _player_left_anim

_player_stop_right::
    .db player_frames
    .dw _sp_athe_00
    .db player_frames
    .dw _sp_athe_01
    .db player_frames
    .dw _sp_athe_02
    .dw nullptr
    .dw _player_stop_right

_player_stop_left::
    .db player_frames
    .dw _sp_athe_06
    .db player_frames
    .dw _sp_athe_07
    .db player_frames
    .dw _sp_athe_08
    .dw nullptr
    .dw _player_stop_left

_player_down_anim::
    .db player_frames
    .dw _sp_athe_12
    .dw nullptr
    .dw _player_down_anim
_player_downLeft_anim::
    .db player_frames
    .dw _sp_athe_18
    .dw nullptr
    .dw _player_downLeft_anim
_player_down_left_anim::
    .db player_frames
    .dw _sp_athe_18
    .db player_frames
    .dw _sp_athe_19
    .db player_frames
    .dw _sp_athe_20
    .dw nullptr
    .dw _player_down_left_anim
_player_down_right_anim::
    .db player_frames
    .dw _sp_athe_12
    .db player_frames
    .dw _sp_athe_13
    .db player_frames
    .dw _sp_athe_14
    .dw nullptr
    .dw _player_down_right_anim
_player_up_anim::
    .db player_frames
    .dw _sp_athe_15
    .dw nullptr
    .dw _player_up_anim
_player_upLeft_anim::
    .db player_frames
    .dw _sp_athe_21
    .dw nullptr
    .dw _player_upLeft_anim
_player_up_left_anim::
    .db player_frames
    .dw _sp_athe_21
    .db player_frames
    .dw _sp_athe_22
    .db player_frames
    .dw _sp_athe_23
    .dw nullptr
    .dw _player_up_left_anim
_player_up_right_anim::
    .db player_frames
    .dw _sp_athe_15
    .db player_frames
    .dw _sp_athe_16
    .db player_frames
    .dw _sp_athe_17
    .dw nullptr
    .dw _player_up_right_anim
_player_death_anim::
    .dw _sp_athe_24
    .dw _sp_athe_25
    .dw _sp_athe_26
    .dw _sp_athe_27
    .dw _sp_athe_28
    .dw _sp_athe_29
    .dw _sp_athe_30
    ;;.dw nullptr
    ;;.dw _player_death_anim

_player_wallLeft_anim::
    .db player_frames
    .dw _sp_athe_31
    .dw nullptr
    .dw _player_wallLeft_anim
_player_wallRight_anim::
    .db player_frames
    .dw _sp_athe_32
    .dw nullptr
    .dw _player_wallRight_anim

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;      Babosa animations
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
_babosa_left_anim::
    .db enemy_frames
    .dw _sp_bab_0
    .db enemy_frames
    .dw _sp_bab_1
    .dw nullptr
    .dw _babosa_left_anim

_babosa_right_anim::
    .db enemy_frames
    .dw _sp_bab_2
    .db enemy_frames
    .dw _sp_bab_3
    .dw nullptr
    .dw _babosa_right_anim

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;      Spider animations
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
_spider_up_anim::
    .db enemy_frames
    .dw _sp_spdr_0
    .db enemy_frames
    .dw _sp_spdr_1
    .db enemy_frames
    .dw _sp_spdr_2
    .db enemy_frames
    .dw _sp_spdr_1
    .dw nullptr
    .dw _spider_up_anim

_spider_down_anim::
    .db enemy_frames
    .dw _sp_spdr_3
    .db enemy_frames
    .dw _sp_spdr_4
    .db enemy_frames
    .dw _sp_spdr_5
    .db enemy_frames
    .dw _sp_spdr_4
    .dw nullptr
    .dw _spider_down_anim


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;     Ghost animations
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
_ghost_left_anim::
    .db enemy_frames
    .dw _sp_ghost_0
    .dw nullptr
    .dw _ghost_left_anim

_ghost_right_anim::
    .db enemy_frames
    .dw _sp_ghost_1
    .dw nullptr
    .dw _ghost_right_anim


_bat_up_anim::
    .db enemy_frames+1
    .dw _sp_bat_0
    .dw nullptr
    .dw _bat_down_anim

_bat_down_anim::
    .db #0x08;enemy_frames
    .dw _sp_bat_1
    .dw nullptr
    .dw _bat_up_anim


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Inicializa al player mirando paradoala izquierda
;; ENTRADAS
;;  
;; SALIDAS
;;  IX - puntero al player
;; MODIFICA
;;  hl
;; PRECONDICIONES
;;    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_animations_facing_left_stopped::
   ld    hl, #_player_stop_left 
   ld    e_animPtr_h(ix), h
   ld    e_animPtr_l(ix), l     
ret