;;	Copyright © CrackTime Studios - 2020
;;
;;	This file is part of Poisoned Escape
;;
;;    Poisoned Escape is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Poisoned Escape is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Poisoned Escape.  If not, see <https://www.gnu.org/licenses/>.

.include "entity.h.s"
.include "cpct_functions.h.s"
.include "game.h.s"
.include "entity_templates.h.s"
.include "animations.h.s"
.include "cpctelera.h.s"
.include "../sys/collisions.h.s"



;;======================================================================================
;; PUBLIC VARIABLES
;;======================================================================================
;  DefineEntity type, cmps, x, y, vx, vy, w, h, sprPtr, AI_status, animTable, animPtr, animSt, prevPtr, collisionMask
m_entities::
   .rept MAX_ENTITIES
      DefineType        e_type_invalid, e_cmps_default
      DefinePhysics     #0x00, #0x00, #0x00, #0x00, 0, 0
      DefineAi          e_ai_st_noAI, nullptr
      DefineAnim        0, #0xDEAD, 0, nullptr, 0
      DefineCollisions  e_type_invalid                                                                            ;; VVV  nulptr OJOOOOOOOOO!!
      ;DefineEntity e_type_invalid, e_cmps_default, #0x00, #0x00, #0x00, #0x00, 0, 0, e_ai_st_noAI, 0, #0xDEAD, 0, nullptr, 0, e_prevPtr_default, e_type_invalid
   .endm
m_type_invalid_at_end::
   .db e_type_invalid
m_next_free_entity::
   .dw #m_entities
m_alive_entities::
   .db 0


;;======================================================================================
;; PRIVATE VARIABLES
;;======================================================================================



;;======================================================================================
;; PUBLIC ROUTINES
;;======================================================================================

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; descripcion
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  -
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_entity_init::
   ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Crea entidad nueva y avanza puntero a siguiente entidad libre
;; ENTRADAS
;;    -
;; SALIDAS
;;    IX y DE - Dirección a la nueva entidad
;; MODIFICA
;;    DE, HL, F
;; PRECONDICIONES
;;    No debe haber llamarse si todas las entidades están vivas
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_entity_create_one_entity::
   ;; Incrementa el número de entidades vivas
   ld    hl, #m_alive_entities      ;; Incrementa número de entidades vivas
   inc  (hl)

   ;; Actualiza el puntero a la nueva posición libre
   ld    de, (#m_next_free_entity)  ;; carga puntero
   ld    ix, (#m_next_free_entity)  ;; IX contiene dirección a la nueva entidad
   ld    hl, #e_sizeof              ;; pone ya el salto en HL a la siguiente
   add   hl, de                     ;; suma obteniendo en HL siguiente entidad y en DE la anterior
   ld    (#m_next_free_entity), hl  ;; guardamos la nueva direccion a la siguiente libre

   ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Aplica a cada entidad la rutina donde apunta el registro HL
;; ENTRADAS
;;  HL - Puntero a rutina para una entidad
;; SALIDAS
;;  -
;; MODIFICA
;;  AF, BC, DE, HL, IX
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_entity_for_all::
   ld    ix, #m_entities         ;; IX apunta a la primera entidad

  man_entity_for_all_loop:
   ld    a, e_type(ix)
   sub   #e_type_invalid
   ret   z                       ;; Si es inválida termina el bucle (shile)

   push  hl ;1*                  ;; guardamos HL que contiene el puntero a rutina
   ld    de, #man_entity_for_all_continue
   push  de ;2*
   jp   (hl);2*                  ;; Si no aplica rutina del puntero en HL

  man_entity_for_all_continue:
   pop   hl ;1*
   ld    de, #e_sizeof
   add   ix, de                  ;; movemos a la siguiente entidad
   jr man_entity_for_all_loop

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Aplica a cada entidad que cumpla la máscara de bits del 
;; registro BC la rutina donde apunta el registro HL
;; ENTRADAS
;;  HL - Puntero a rutina para una entidad
;;  C - Signatura (máscara de bits)
;; SALIDAS
;;  -
;; MODIFICA
;;  AF, BC, DE, HL, IX
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_entity_for_all_matching::
   ld    ix, #m_entities            ;; IX apunta a la primera entidad      

  man_entity_for_all_matching_loop:
   ld    a, e_type(ix)
   sub   #e_type_invalid
   ret   z                          ;; Si es inválida termina el bucle (while)

   ;; Es válida
   ld    a, e_cmps(ix)
   and   c                          ;; al hacer and y cumplir la máscara =>   a == c
   sub   c                          ;; Por lo que al restar c => (a == 0) => (z=1=cumple la signatura)

   jr    nz, man_entity_for_all_matching_continue  ;; No cumple la signatura
      ;; Cumple la signatura
      push  hl ;1*                  ;; guardamos HL que contiene el puntero a rutina
      push  bc ;2*
      ld    de, #man_entity_for_all_matching_continue_from_routine
      push  de ;3*
      jp  (hl) ;3*                  ;; Si no aplica rutina del puntero en HL
      man_entity_for_all_matching_continue_from_routine:
         pop   bc ;2*                     ;; recuperamos máscara y puntero a rutina
         pop   hl ;1*
  man_entity_for_all_matching_continue:
   ld    de, #e_sizeof
   add   ix, de                     ;; movemos a la siguiente entidad
   jr   man_entity_for_all_matching_loop

ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Elimina la componente movable de las entidades
;; ENTRADAS
;;  IX - Puntero a entidad 
;; SALIDAS
;;  -
;; MODIFICA
;;  -
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_entity_disable_cmps_movable::
   ld a, e_cmps(ix)
   ld d, #e_cmps_movable
   sub d
   ld e_cmps(ix), a
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Aplica a cada entidad que cumpla la máscara de bits del 
;; registro BC la rutina donde apunta el registro HL
;; ENTRADAS
;;  HL - Puntero a rutina para una entidad
;;  C - Signatura (máscara de bits)
;; SALIDAS
;;  -
;; MODIFICA
;;  AF, BC, DE, HL, IX
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_entity_for_all_pairs_matching::
   ld    ix, #m_entities            ;; IX apunta a la primera entidad      

  man_entity_for_all_pairs_matching_first_loop:
   ld    a, e_type(ix)
   sub   #e_type_invalid
   ret   z                          ;; Si es inválida termina el bucle (while)

   ;; Es válida
   ld    a, e_cmps(ix)
   and   c                          ;; al hacer and y cumplir la máscara =>   a == c
   sub   c                          ;; Por lo que al restar c => (a == 0) => (z=1=cumple la signatura)

   jr    nz, man_entity_for_all_pairs_matching_first_continue  ;; No cumple la signatura
      ;; cumple la signatura
      ;; iy = ix + size
      push  ix
      pop   iy
      ld    de, #e_sizeof
      add   iy, de                     ;; movemos a la siguiente entidad
     man_entity_for_all_pairs_matching_second_loop:
      ld    a, e_type(iy)
      sub   #e_type_invalid
      jr    z, man_entity_for_all_pairs_matching_first_continue ;; Si es inválida termina el bucle (while)

      ;; Es válida
      ld    a, e_cmps(iy)
      and   c                          ;; al hacer and y cumplir la máscara =>   a == c
      sub   c                          ;; Por lo que al restar c => (a == 0) => (z=1=cumple la signatura)

      jr    nz, man_entity_for_all_pairs_matching_second_continue  ;; No cumple la signatura
         ;; Cumple la signatura
         push  hl ;1*                  ;; guardamos HL que contiene el puntero a rutina
         push  bc ;2*
         ld    de, #man_entity_for_all_pairs_matching_continue_from_routine
         push  de ;3*
         jp  (hl) ;3*                  ;; Si no aplica rutina del puntero en HL
         man_entity_for_all_pairs_matching_continue_from_routine:
            pop   bc ;2*                     ;; recuperamos máscara y puntero a rutina
            pop   hl ;1*
     man_entity_for_all_pairs_matching_second_continue:
      ld    de, #e_sizeof
      add   iy, de                     ;; movemos a la siguiente entidad
      jr   man_entity_for_all_pairs_matching_second_loop
      
  man_entity_for_all_pairs_matching_first_continue:
   ld    de, #e_sizeof
   add   ix, de                     ;; movemos a la siguiente entidad
   jr   man_entity_for_all_pairs_matching_first_loop

   ret




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Indica si queda espacio libre con el flag Z=0. Si Z=1, no hay
;; espacio libre
;; ENTRADAS
;;    - 
;; SALIDAS
;;    Z - Si Z=1 no queda espacio,
;;    A - Espacio libre
;; MODIFICA
;;    AF, BC
;; PRECONDICIONES
;;    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_entity_freeSpace::
   ld     a, (#m_alive_entities)    ;; Carga en A el número de entidades vivas para copiarlo a B
   ld     b, a                      ;; Copia en B el número de entidades vivas
   ld     a, #MAX_ENTITIES          ;; Carga en A el número de entidades máximo
   sub    b                         ;; Calcula la diferencia para saber cuántas crear
   ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Marca la entidad para destruir => añade bit e_type_dead
;; ENTRADAS
;;    IX - Dirección de la entidad
;; SALIDAS
;;    -
;; MODIFICA
;;    AF
;; PRECONDICIONES
;;    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;   
man_entity_set4destruction::
   ld a, e_type(ix)
   or #e_type_dead                  ;; Añade primer bit de la izquierda (b7=1 => 0x80 = 0b10000000)
   ld e_type(ix), a                 ;; Actualiza el tipo
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Destruye la entidad, copiando la última en ella, marcando
;; la última como inválida y retrocediendo el puntero
;; ENTRADAS
;;    DE - Dirección de la entidad a destruir
;; SALIDAS
;;    -
;; MODIFICA
;;    BC, HL, AF, IY
;; PRECONDICIONES
;;    Debe ser una no inválida
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_destroy_one_entity::
   ;; Calculamos nuevo puntero a entidad libre y lo actualizamos           
   ld   hl, (#m_next_free_entity)     ;; dirección a la variable que contiene puntero a siguiente libre
   ld   bc, #-e_sizeof
   add  hl, bc                        ;; Posicionamos en la penúltima
   ld   (#m_next_free_entity), hl     ;; Guardamos nueva posición
  
   ;; Copiamos la penúltima a la posición de la entidad a borrar (OJO: si no es la única, no la última)
   ld    a, (#m_alive_entities)
   dec   a
   jr    z, continue_destroy_one_entity;; Para no copiar la memoria de la misma
      ld   bc, #e_sizeof               ;; Indicamos cuantos bytes copiar    
      ldir                             ;; OJO: modifica AF, BC, DE, HL. Recibe DE
  
  ;; Marcamos como inválida la siguiente libre y decrementamos el número de entidades vivas
  continue_destroy_one_entity:
   ld    iy, (#m_next_free_entity)
   ld    e_type(iy), #e_type_invalid   ;; marcamos como inválida

   ld   hl, #m_alive_entities          ;; Decrementamos contador de entidades vivas
   dec  (hl)                           ;; OJO: las no vivas no están marcadas como inválidas
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Actualiza las entidades borrando las que estaban marcadas para
;; ser destruidas
;; ENTRADAS
;;    - 
;; SALIDAS
;;    -
;; MODIFICA
;;    DE, HL, AF, BC, IX, IY
;; PRECONDICIONES
;;    -
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_entity_update::
   ld    ix, #m_entities                             ;; HL apunta a la primera entidad
  man_entity_update_loop:                             ;; recorre las entidades válidas
   ;; mientras no sea inválida
   ld     a, e_type(ix) 
   cp   #e_type_invalid                               ;; si (a = 0) => (0 - 0) = 0 => z = 1 => inválida
   jr     z, man_entity_update_end                    ;; Si es inválida, sale del while                      
      ;; Si está muerta la destruye
      and    #e_type_dead                             ;; Si no comprueba si está muerta
      jr     z, man_entity_update_loop_continue_alive ;; Si el and activa Z está viva
         ;; Está muerta, por lo que la destruye
         ld__d_ixh
         ld__e_ixl
         call  man_destroy_one_entity                 ;; OJO: modifica AF, DE, BC, HL
         jr    #man_entity_update_loop_continue       ;; No incrementa el puntero
      man_entity_update_loop_continue_alive:          ;; si está viva solo incrementa puntero
         ld    de, #e_sizeof
         add   ix, de
      man_entity_update_loop_continue:                ;; final del while, vuelve arriba 
         jr    man_entity_update_loop
  man_entity_update_end:
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Marca la entidad para destruir => añade bit e_type_dead
;; ENTRADAS
;;    IX - Dirección de la entidad
;; SALIDAS
;;    -
;; MODIFICA
;;    AF, HL
;; PRECONDICIONES
;;    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
man_entity_set4destruction_all_entities::
   ld hl, #man_entity_set4destruction
   call man_entity_for_all
   ret

;;======================================================================================
;; PRIVATE ROUTINES
;;======================================================================================

