;;	Copyright © CrackTime Studios - 2020
;;
;;	This file is part of Poisoned Escape
;;
;;    Poisoned Escape is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Poisoned Escape is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Poisoned Escape.  If not, see <https://www.gnu.org/licenses/>.

.module ENTITY_TEMPLATES

;;================================================
;; CONSTANTS
;;================================================

;;    ---------------Animation--------------------------
;;    1B Frames for each sprite
;;    1B Animation table   (anim_Table)
;;    2B Animation Pointer (anim_ptr)
;;    1B Animation status  (anim_st)
;;

;; Order
mType_pos         = 0
mPhysics_pos      = mType_pos    + mType_size
mAi_pos           = mPhysics_pos + mPhysics_size
mAnim_pos         = mAi_pos      + mAi_size
mCollisions_pos   = mAnim_pos    + mAnim_size
e_sizeof          = mCollisions_pos + mCollisions_size

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.macro DefineType _type, _cmps
   .db _type      ;; Type
   .db _cmps      ;; Components
.endm
e_type  = 0 + mType_pos
e_cmps  = 1 + mType_pos
mType_size = 2


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.macro DefinePhysics _x, _y, _vx, _vy, _w, _h
   .db _x         ;; Position 'X' rounded of entity
   .db _y         ;; Position 'Y' rounded of entity
   .dw (_x<<8)    ;; Position 'X' of entity
   .dw (_y<<8)    ;; Position 'Y' of entity
   .dw _vx        ;; Speed on 'X'
   .dw _vy        ;; Speed on 'Y'
   .db _w         ;; Width (ancho)
   .db _h         ;; Height (altura)
.endm
e_x_rounded = 0 + mPhysics_pos
e_y_rounded = 1 + mPhysics_pos
e_xf        = 2 + mPhysics_pos
e_x         = 3 + mPhysics_pos
e_yf        = 4 + mPhysics_pos
e_y         = 5 + mPhysics_pos
e_vxf       = 6 + mPhysics_pos
e_vx        = 7 + mPhysics_pos
e_vyf       = 8 + mPhysics_pos
e_vy        = 9 + mPhysics_pos
e_w         = 10 + mPhysics_pos
e_h         = 11 + mPhysics_pos
mPhysics_size = 12


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.macro DefineAI _ai_st, _ai_movPtr
   .db 0x00, 0x00 ;; AI - aim (ai_aim_init x, ai_aim_init y)
   .db 0x00, 0x00 ;; AI - aim (ai_aim_end x, ai_aim_end y)
   .dw _ai_movPtr ;; AI - routine that depends on axis
   .db _ai_st     ;; AI-satus (ai status)
.endm
e_ai_aim_init_x   = 0 + mAi_pos
e_ai_aim_init_y   = 1 + mAi_pos 
e_ai_aim_end_x    = 2 + mAi_pos
e_ai_aim_end_y    = 3 + mAi_pos 
e_ai_movPtr_l     = 4 + mAi_pos   ;; Rutine for axis AI
e_ai_movPtr_h     = 5 + mAi_pos   ;; Rutine for axis AI
e_ai_st           = 6 + mAi_pos   ;; 1 -> to init, 2-> to pos end
mAi_size = 7


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.macro DefineAnim  _animFPS, _sprPtr, _animTable, _animPtr, _animSt
   .db _animFPS   ;; Frames for each sprite
   .dw _sprPtr    ;; Sprite Pointer
   .db _animTable ;; Animation table 
   .dw _animPtr   ;; Animation Pointer (anim ptr)
   .db _animSt    ;; Animation status  
   .dw 0x0000     ;; prevPtr
.endm
 
e_animFPS   = 0 + mAnim_pos        ;;----------------- Frames for each sprite
e_sprPtr_l  = 1 + mAnim_pos
e_sprPtr_h  = 2 + mAnim_pos
e_animTable = 3 + mAnim_pos        ;;-----------------  Animation table
e_animPtr_l = 4 + mAnim_pos        ;;-----------------  Animation
e_animPtr_h = 5 + mAnim_pos        ;;-----------------
e_animSt    = 6 + mAnim_pos        ;; ----------------  Animation status
e_prevPtr_l = 7 + mAnim_pos
e_prevPtr_h = 8 + mAnim_pos
mAnim_size  = 9


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.macro DefineCollisions _coll
   .db _coll
   .db 0 ;; Collisiones iniciales con tiles, solo para el player quien con esto debería empezar sobre un suelo
.endm
e_collides_against   = 0 + mCollisions_pos
e_tile_collisions    = 1 + mCollisions_pos
mCollisions_size     = 2




;; e_types
e_type_invalid      = #0x00
e_type_player       = #0x01
e_type_enemy        = #0x02
e_type_end          = #0x04
e_type_easter_egg_end = #0x08
e_type_default      = #0x7F
e_type_dead         = #0x80


;; e_cmps
e_cmps_default       = #0x00
e_cmps_renderable    = #0x01
e_cmps_movable       = #0x02
e_cmps_input         = #0x04
e_cmps_collisionable = #0x08
e_cmps_tile_collisionable = #0x10
e_cmps_anime         = #0x20
e_cmps_ai            = #0x40
e_cmps_gravitable    = #0x80

e_prevPtr_default   = #0x0000

;; entity AI-status enum
e_ai_st_noAI = #0x00
e_ai_st_move_to = #0x01




;;================================================
;; PUBLIC VARIABLES
;;================================================

.globl player_tmpl
.globl enemy_tmpl
.globl babosa_tmpl
.globl ghost_tmpl
.globl bat_tmpl
.globl end_tmpl

;;================================================
;; PRIVATE VARIABLES
;;================================================


;;================================================
;; PUBLIC ROUTINES
;;================================================


;;================================================
;; PRIVATE ROUTINES
;;================================================