;;	Copyright © CrackTime Studios - 2020
;;
;;	This file is part of Poisoned Escape
;;
;;    Poisoned Escape is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Poisoned Escape is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Poisoned Escape.  If not, see <https://www.gnu.org/licenses/>.

.include "game.h.s"
.include "menu.h.s"
.include "cpct_functions.h.s"
.include "entity.h.s"
.include "../sys/physics.h.s"
.include "../sys/render.h.s"
.include "../assets/assets.h.s"
.include "entity_templates.h.s"
.include "sys/animation.h.s"
.include "animations.h.s"
.include "../sys/input.h.s"
.include "../sys/collisions.h.s"
.include "../assets/maps/tilemap1.h.s"
.include "sys/ai_control.h.s"
.include "assets/screens/screennextlevel_z.h.s"
.include "musics.h.s"
.include "cpctelera.h.s"
.include "assets/screens/screenfinished_z.h.s"
;;======================================================================================
;; PUBLIC VARIABLES
;;======================================================================================

RUN_PROGRAM    == 0x4000
tilemapSize    == 500
tilemapPtr     == RUN_PROGRAM-tilemapSize
tilemapPtrEnd  == tilemapPtr+tilemapSize-1  
currentTilemap:: .dw nullptr

bX = 4
bY = 8
m_game_levels::
   .dw man_game_level_easy_5_carlos    ;1 
   .dw man_game_level_easy_1_adrian    ;2 
   .dw man_game_level_easy_2_adrian    ;3 
   .dw man_game_level_easy_3_adrian    ;4 
   .dw man_game_level_easy_1_sam       ;5    
   .dw man_game_level_easy_6_carlos    ;6    ;; Spiders 1
   .dw man_game_level_2                ;7    ;; Spiders 2
   .dw man_game_level_easy_8_carlos    ;8    ;; Ghosts
   .dw man_game_level_1                ;9    ; Easter Egg
   .dw man_game_level_3                ;10
   .dw man_game_level_medium_7_carlos  ;11
   .dw man_game_level_medium_1_adrian  ;12
   .dw man_game_level_easy_3_carlos    ;13
   .dw man_game_level_medium_1_sam     ;14
   .dw man_game_level_medium_4_carlos  ;15
   .dw man_game_level_medium_1_carlos  ;16
   .dw man_game_level_hard_2_carlos    ;17
   .dw man_game_level_medium_2_sam     ;18
   .dw man_game_level_hard_1_sam       ;19   ;
   .dw man_game_level_hard_1_adrian    ;20   ;; Tiene string 'Final level'
   .dw man_game_level_end              ;; Cargamos el nivel de final de juego  

gameState:: .db game_stopped     ;; 0 -> Not playing     1 -> Playing   -> 2 Next level
 
currentLevel:: .dw nullptr
levelCounter:: .db 0

deathcounterUnits:: .db 0
deathcounterTens:: .db 0

deathcounterHundreds:: .db 0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pause_next_level = #80

pause_end_game = 10*25
pause_end_game2 = 1*25


NUM_LEVELS = 25                ;; Cte. que dicta el No. de Niveles en el juego

string_level: .asciz "Level";
string_num: .asciz "0"
string_numm = #48

onEasterEgg:: .db 0     ;; 0 - undetected, 1 - detected, 2 - finished
string_easter_egg_init = .
string_easter_egg: .asciz "Looks like he never saved the princess"
string_easter_egg_end = .
string_easter_egg_size = string_easter_egg_end - string_easter_egg_init

string_final_level_init = .
string_final_level: .asciz "Final level"
string_final_level_end = .
string_final_level_size = string_final_level_end - string_final_level_init

string_pause: .asciz "|| Pause"
string_clear_pause: .asciz "        "


string_deaths: .asciz "Deaths:" 

string_total_deaths: .asciz "Total Deaths:"

;;======================================================================================
;; PRIVATE VARIABLES
;;======================================================================================

playerOnFloor::
   .db 0

;;======================================================================================
;; PUBLIC ROUTINES
;;======================================================================================



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Inicializa el juego
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  -
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_init::
   ld de , #_song_menu
   call man_music_init_music_25

   ld    e, #2                               ;; L = Foreground PEN (5)
   ld    d, #0                               ;; H = Background PEN (0)
   
   call cpct_setDrawCharM1_asm               ;; Set draw char colours

   ld hl, #gameState                         ;; Inicializamos el estado de juego a 0
   ld (hl), #game_stopped
   ld hl, #levelCounter                      ;; Contador de nivel a 0
   ld (hl), #0               
   ld hl, #deathcounterUnits
   ld (hl), #0
   ld hl, #deathcounterUnits
   ld (hl), #0
   ld hl, #deathcounterUnits
   ld (hl), #0       

   call man_menu_load_menu
ret

;.globl cpct_clearScreen_asm
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Carga el nivel 0 del juego
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  AF, BC, DE, HL, IX
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_level_end::
   ld     hl, #_screenfinished_z_end
   ld    de, #0xFFFF

   call cpct_zx7b_decrunch_s_asm

   ;call sys_render_update

   ld d, #pause_end_game2                  
   call sys_render_wait

   ;ld d, #pause_end_game2                  
   ;call sys_render_wait


   ld  de, #0xC000
   ld  a, #backgroundColor ;(1B A ) colour_pattern	1-byte colour pattern (in screen pixel format) to fill the box with
   ld  c, #65          ;(1B C ) width	Box width in bytes [1-64] (Beware!  not in pixels!)
   ld  b, #screen_height         ;(1B B ) height	Box height in bytes (>0)
   
   ;ld  de, #0xC400
   ;ld  a, #backgroundColor ;(1B A ) colour_pattern	1-byte colour pattern (in screen pixel format) to fill the box with
   ;ld  c, #65          ;(1B C ) width	Box width in bytes [1-64] (Beware!  not in pixels!)
   ;ld  b, #screen_height         ;(1B B ) height	Box height in bytes (>0)
   
   ;call cpct_drawSolidBox_asm
   ;ld d, #pause_end_game2                  
   ;call sys_render_wait
   ;call cpct_waitVSYNC_asm

   ;ld a, #backgroundColor
   ;cpct_clearScreen_asm 

   call man_game_print_total_deaths
   ld d, #pause_end_game                  
   call sys_render_wait

   ld  de, #0xC000
   ld  a, #backgroundColor ;(1B A ) colour_pattern	1-byte colour pattern (in screen pixel format) to fill the box with
   ld  c, #screen_width          ;(1B C ) width	Box width in bytes [1-64] (Beware!  not in pixels!)
   ld  b, #screen_height         ;(1B B ) height	Box height in bytes (>0)
   call cpct_drawSolidBox_asm
   
   
   
   call man_game_init




ret


.globl _tilemap_medium_1_Carlos_end
man_game_level_medium_1_carlos::

   ld hl,   #gameState
   ld (hl), #game_playing

   ld   hl, #_tilemap_medium_1_Carlos_end
   call man_game_change_tilemap

   ld   d, #1
   ld   e, #6
   call man_game_print_level


   CreateEntityXY player, bX*1, bY*5
   call man_animations_facing_left_stopped

   ; End entity
   CreateEntityXY end, bX*18+2, bY*12           
   call sys_physics_round_XY  ;; OJO: END lo necesita ya que esto solo se calcula en físicas


   ;;bX = 4; ;;bY = 8

   ;; third entity   (AI)
   CreateEntityXY enemy, bX*3, bY*7-1
   ChangeAILimits y, bX*6+1, bY*21
   
   ;; third entity   (AI)
   CreateEntityXY enemy, bX*4-1, bY*15
   ChangeAILimits y, bX*6+1, bY*21

   ;; third entity   (AI)
   CreateEntityXY bat, bX*10+2, bY*5+2
   ChangeAILimits x, bX*8, bY*5+3
   ChangeAILimits y, bX*9+2, bY*6+2


   ;; third entity   (AI)
   CreateEntityXY babosa, bX*10, bY*23
   ChangeAILimits x, bX*4+2, bX*11
   
   call man_game_print_deaths
   
ret



.globl _tilemap_hard_2_Carlos_end
man_game_level_hard_2_carlos::

   ld hl,   #gameState
   ld (hl), #game_playing

   ld   hl, #_tilemap_hard_2_Carlos_end
   call man_game_change_tilemap

   ld   d, #1
   ld   e, #7
   call man_game_print_level


   CreateEntityXY player, bX*1+1, bY*6

   ; End entity
   CreateEntityXY end, bX*19+2, bY*6          
   call sys_physics_round_XY  ;; OJO: END lo necesita ya que esto solo se calcula en físicas


   ;;; enemy entity   (AI)
   CreateEntityXY bat, bX*10, bY*6
   ChangeAILimits x, bX*10, bX*17
   ChangeAILimits y, bY*3, bY*6-1
   
   ;; enemy entity   (AI)
   CreateEntityXY ghost, bX*7-2, bY*5
   ChangeAILimits x, bX*2+2, bY*4-2
   ChangeAILimits y, bX*6+1, bY*6+2

   CreateEntityXY ghost, bX*15, bY*10
   ChangeAILimits x, bX*15, bX*18-1
   ChangeAILimits y, bY*10, bY*16

   ;;; enemy entity   (AI)
   CreateEntityXY babosa, bX*5 , bY*8
   ChangeAILimits x, bX*2, bX*6

   ;;; enemy entity   (AI)
   CreateEntityXY babosa, bX*10 , bY*20
   ChangeAILimits x, bX*10, bX*12


    ;;; enemy entity   (AI)
   CreateEntityXY enemy, bX*3 , bY*18
   ChangeAILimits y, bY*11, bY*20-2

   call man_game_print_deaths


   
ret

.globl _tilemap_easy_3_Carlos_end
man_game_level_easy_3_carlos::

   ld hl,   #gameState
   ld (hl), #game_playing

   ld   hl, #_tilemap_easy_3_Carlos_end
   call man_game_change_tilemap

   ld   d, #1
   ld   e, #3
   call man_game_print_level


   CreateEntityXY player, bX*17, bY*22
   call man_animations_facing_left_stopped

   ; End entity
   CreateEntityXY end, bX*18+2, bY*4         
   call sys_physics_round_XY  ;; OJO: END lo necesita ya que esto solo se calcula en físicas
   

   ;; enemy entity   (AI)
   CreateEntityXY ghost, bX*7, bY*5
   ChangeAILimits x, bX*2+2, bX*9-2
   ChangeAILimits y, bY*3+1, bY*6+2

   CreateEntityXY enemy, bX*12, bY*9-2
   ChangeAILimits y, bY*3, bY*9-2

   ;;;; enemy entity   (AI)
   CreateEntityXY babosa, bX*2 , bY*23
   ChangeAILimits x, bX*2, bX*7

   call man_game_print_deaths

   
ret


.globl _tilemap_easy_5_Carlos_end
man_game_level_easy_5_carlos::
   

   ld hl,   #gameState
   ld (hl), #game_playing

   ld   hl, #_tilemap_easy_5_Carlos_end
   call man_game_change_tilemap

   ld   d, #0
   ld   e, #1
   call man_game_print_level


   CreateEntityXY player, bX*1+1, bY*22

   ; End entity
   CreateEntityXY end, bX*19+2, bY*22        
   call sys_physics_round_XY  ;; OJO: END lo necesita ya que esto solo se calcula en físicas
   call man_game_print_deaths

ret


.globl _tilemap_easy_8_Carlos_end
man_game_level_easy_8_carlos::

   ld hl,   #gameState
   ld (hl), #game_playing

   ld   hl, #_tilemap_easy_8_Carlos_end
   call man_game_change_tilemap

   ld   d, #0
   ld   e, #8
   call man_game_print_level


   CreateEntityXY player, bX*1+1, bY*12

    ;;; enemy entity   (AI)
   CreateEntityXY ghost, bX*1, bY*8
   ChangeAILimits x, bX*1+2, bX*9+2
   ChangeAILimits y, bY*3, bY*10

    ;;; enemy entity   (AI)
   CreateEntityXY ghost, bX*7, bY*8
   ChangeAILimits x, bX*7+2, bX*12+2
   ChangeAILimits y, bY*3, bY*9

    ;;; enemy entity   (AI)
   CreateEntityXY ghost, bX*7, bY*6
   ChangeAILimits x, bX*10+2, bX*18
   ChangeAILimits y, bY*3, bY*13


   

   ; End entity
   CreateEntityXY end, bX*19+2, bY*13        
   call sys_physics_round_XY  ;; OJO: END lo necesita ya que esto solo se calcula en físicas
   call man_game_print_deaths

ret


.globl _tilemap_easy_6_Carlos_end
man_game_level_easy_6_carlos::

   ld hl,   #gameState
   ld (hl), #game_playing

   ld   hl, #_tilemap_easy_6_Carlos_end
   call man_game_change_tilemap

   ld   d, #0
   ld   e, #6
   call man_game_print_level


   CreateEntityXY player, bX*2, bY*10

   ; End entity
   CreateEntityXY end, bX*19+2, bY*12        
   call sys_physics_round_XY  ;; OJO: END lo necesita ya que esto solo se calcula en físicas
   

   CreateEntityXY enemy, bX*5+1, bY*12-4
   ChangeAILimits y, bY*3, bY*15-4

   CreateEntityXY enemy, bX*13+1, bY*3
   ChangeAILimits y, bY*3, bY*15-4
   call man_game_print_deaths
   
ret


.globl _tilemap_medium_7_Carlos_end
man_game_level_medium_7_carlos::

   ld hl,   #gameState
   ld (hl), #game_playing

   ld   hl, #_tilemap_medium_7_Carlos_end
   call man_game_change_tilemap

   ld   d, #1
   ld   e, #1
   call man_game_print_level


   CreateEntityXY player, bX*1+2, bY*15

   ; End entity
   CreateEntityXY end, bX*19+2, bY*18         
   call sys_physics_round_XY  ;; OJO: END lo necesita ya que esto solo se calcula en físicas
   

   ;;;;; enemy entity   (AI)
   CreateEntityXY babosa, bX*4 , bY*12
   ChangeAILimits x, bX*3, bX*5

   ;;;;; enemy entity   (AI)
   CreateEntityXY babosa, bX*6 , bY*8
   ChangeAILimits x, bX*6, bX*8

   CreateEntityXY enemy, bX*11+2, bY*11
   ChangeAILimits y, bY*3, bY*21-2


   ;CreateEntityXY enemy, bX*7, bY*14-4
   ;ChangeAILimits y, bY*3, bY*14-4
;
   ;CreateEntityXY enemy, bX*10+1, bY*11
   ;ChangeAILimits y, bY*3, bY*16-4
;
   ;;; enemy entity   (AI)
   ;CreateEntityXY bat, bX*6, bY*15-1
   ;ChangeAILimits x, bX*1, bX*6
   ;ChangeAILimits y, bY*13, bY*19-2
;
  ;
   ;;; enemy entity   (AI)
   CreateEntityXY ghost, bX*7, bY*19
   ChangeAILimits x, bX*6+2, bX*17+2
   ChangeAILimits y, bY*16, bY*19

   call man_game_print_deaths
ret

   

.globl _tilemap_medium_1_adrian_end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  AF, BC, DE, HL, IX
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_level_medium_1_adrian::

   ld hl,   #gameState
   ld (hl), #game_playing

   ld   hl, #_tilemap_medium_1_adrian_end
   call man_game_change_tilemap

   ld   d, #1
   ld   e, #2
   call man_game_print_level

   CreateEntityXY player, bX*8  , bY*22
   call man_animations_facing_left_stopped

   CreateEntityXY    babosa, bX*10   , bY*7
   ChangeAILimits      x, bX*10   , bX*12+2

   CreateEntityXY    bat, bX*1   , bY*14
   ChangeAILimits      y, bY*14  , bY*17
   ChangeAILimits      x, bX*1   , bX*3+2

   CreateEntityXY    bat, bX*6   , bY*10
   ChangeAILimits      y, bY*10  , bY*13
   ChangeAILimits      x, bX*5+2 , bX*8

   CreateEntityXY    bat, bX*1   , bY*6
   ChangeAILimits      y, bY*6   , bY*9
   ChangeAILimits      x, bX*1   , bX*3+2

   CreateEntityXY    bat, bX*13   , bY*17
   ChangeAILimits      y, bY*17     , bY*20-1
   ChangeAILimits      x, bX*13   , bX*16+2

   ;; End entity
   CreateEntityXY    end, bX*19+2 , bY*22           
   call sys_physics_round_XY  ;; OJO: END lo necesita ya que esto solo se calcula en físicas
   call man_game_print_deaths
ret

   
ret

.globl _tilemap_medium_4_Carlos_end
man_game_level_medium_4_carlos::

   ld hl,   #gameState
   ld (hl), #game_playing

   ld   hl, #_tilemap_medium_4_Carlos_end
   call man_game_change_tilemap

   ld   d, #1
   ld   e, #5
   call man_game_print_level


   CreateEntityXY player, bX*1+1, bY*18

   ; End entity
   CreateEntityXY end, bX*18+2, bY*5         
   call sys_physics_round_XY  ;; OJO: END lo necesita ya que esto solo se calcula en físicas
   
   CreateEntityXY enemy, bX*7, bY*14-4
   ChangeAILimits y, bY*3, bY*14-4

   CreateEntityXY enemy, bX*10+1, bY*11
   ChangeAILimits y, bY*3, bY*16-4

   ;; enemy entity   (AI)
   CreateEntityXY bat, bX*6, bY*15-1
   ChangeAILimits x, bX*1, bX*6
   ChangeAILimits y, bY*13, bY*19-2

  
   ;; enemy entity   (AI)
   CreateEntityXY ghost, bX*13, bY*17
   ChangeAILimits x, bX*11, bX*16+2
   ChangeAILimits y, bY*14, bY*18
;
   ;CreateEntityXY enemy, bX*12, bY*9-2
   ;ChangeAILimits y, bY*3, bY*9-2
;
   ;;;;; enemy entity   (AI)
   ;CreateEntityXY babosa, bX*2 , bY*23
   ;ChangeAILimits x, bX*2, bX*7

   call man_game_print_deaths
   
ret


.globl _tilemap1_end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Carga el nivel 0 del juego
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  AF, BC, DE, HL, IX
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_level_1::

   ld hl,   #gameState
   ld (hl), #game_playing

   ld   hl, #_tilemap1_end
   call man_game_change_tilemap

   ld   d, #0
   ld   e, #9
   call man_game_print_level

   ld   a, (#onEasterEgg)
   or   a
   jr   z, #playerOriginalPos
      CreateEntityXY player, 4*0+2, 8*6
      ld a, #0
      ld (#onEasterEgg), a
      jp playerOriginalPosContinue
  playerOriginalPos:
   CreateEntityXY player, 4*1+1, 8*14
  playerOriginalPosContinue:

   ;; Second entity   (AI)
   CreateEntityXY enemy, 20, 160
   ChangeAILimits y, 100, 180


   ;; third entity   (AI)
   CreateEntityXY babosa, 48, 104
   ChangeAILimits x, 44, 53

   
   CreateEntityXY end, 4*0, 8*6
   ld e_type(ix), #e_type_easter_egg_end
   ld e_w(ix), #1
   call sys_physics_round_XY

   ; End entity
   CreateEntity end           ;; Si queremos XY tiene que llamarse CreateEntityXY y descomentar la línea de abajo
   ;call sys_physics_round_XY  ;; OJO: END lo necesita ya que esto solo se calcula en físicas

   call man_game_print_deaths
ret

.globl _tilemap2_end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Carga el nivel 3 del juego
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  AF, BC, DE, HL, IX
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_level_2::

   ld hl,   #gameState
   ld (hl), #game_playing

   ld   hl, #_tilemap2_end
   call man_game_change_tilemap

   ld   d, #0
   ld   e, #7
   call man_game_print_level


   CreateEntityXY player, bX*4   , bY*6
   call man_animations_facing_left_stopped

   ;;; First entity   (AI)
   ;CreateEntityXY  enemy, bX*2   , bY*8
   ;ChangeAILimits      y, bY*8   , bY*18

   ;; Second entity   (AI)
   CreateEntityXY  enemy, bX*8+2 , bY*15
   ChangeAILimits      y, bY*8   , bY*15


   ;; third entity   (AI)
   CreateEntityXY  enemy, bX*12+2, bY*4
   ChangeAILimits      y, bY*4   , bY*11
   
   ; End entity
   CreateEntityXY   end, (screen_width-2), (screen_height-5*8)           
   call sys_physics_round_XY  ;; OJO: END lo necesita ya que esto solo se calcula en físicas
   call man_game_print_deaths
ret

.globl _tilemap3_end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Carga el nivel 2 del juego
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  AF, BC, DE, HL, IX
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_level_3::
   ld hl, #gameState
   ld (hl), #game_playing      

   ld   hl, #_tilemap3_end
   call man_game_change_tilemap

   ld   d, #1
   ld   e, #0
   call man_game_print_level
   
   CreateEntityXY player, bX*1+1 , bY*6

   ;; Second entity   (AI)
   CreateEntityXY enemy, 20, 160
   ChangeAILimits y, 100, 180

   ;; Third entity   (AI)
   CreateEntityXY enemy, 4*15+3, bY*12
   ChangeAILimits y, bY*8, 152
   
   ;; Fourth entity   (AI)
   CreateEntityXY babosa, 54, 96
   ChangeAILimits x, 48, 58

   ;; End entity
   CreateEntityXY end, 32, 64
   call sys_physics_round_XY        ;; OJO: END lo necesita ya que esto solo se calcula en físicas
   call man_game_print_deaths
ret

.globl _tilemap_easy_1_sam_end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Carga el nivel 2 del juego
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  AF, BC, DE, HL, IX
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_level_easy_1_sam::
   ld hl, #gameState
   ld (hl), #game_playing      

   ld   hl, #_tilemap_easy_1_sam_end
   call man_game_change_tilemap

   ld   d, #0
   ld   e, #5
   call man_game_print_level
   
   CreateEntityXY player, bX*1+1, bY*12
   
   ;; third entity   (AI)
   CreateEntityXY babosa, bX*6  , bY*18
   ChangeAILimits x, bX*6  , bX*10

   ;; End entity
   CreateEntityXY end, bX*19+2 , bY*11
   call sys_physics_round_XY        ;; OJO: END lo necesita ya que esto solo se calcula en físicas
   call man_game_print_deaths
ret

.globl _tilemap_medium_1_sam_end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Carga el nivel 2 del juego
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  AF, BC, DE, HL, IX
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_level_medium_1_sam::
   ld hl, #gameState
   ld (hl), #game_playing      

   ld   hl, #_tilemap_medium_1_sam_end
   call man_game_change_tilemap

   ld   d, #1
   ld   e, #4
   call man_game_print_level
   
   CreateEntityXY player, bX*1, bY*6
   
   ;; third entity   (AI)
   CreateEntityXY babosa, bX*9  , bY*7
   ChangeAILimits x, bX*8  , bX*11

   CreateEntityXY babosa, bX*12  , bY*10
   ChangeAILimits x, bX*12 , bX*16 

   CreateEntityXY  bat, bX*17  , bY*4
   ChangeAILimits      y, bY*5  , bY*8
   ChangeAILimits      x, bX*15 , bX*18

   CreateEntityXY  ghost, bX*5  , bY*16
   ChangeAILimits      y, bY*15  , bY*18
   ChangeAILimits      x, bX*5 , bX*14

   CreateEntityXY babosa, bX*5  , bY*20     ;; down
   ChangeAILimits x, bX*5  , bX*7

   CreateEntityXY babosa, bX*17  , bY*20
   ChangeAILimits x, bX*15  , bX*17

   ;; End entity
   CreateEntityXY end, bX*19+2 , bY*17
   call sys_physics_round_XY        ;; OJO: END lo necesita ya que esto solo se calcula en físicas
   call man_game_print_deaths
ret

.globl _tilemap_medium_2_sam_end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Carga el nivel 2 del juego
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  AF, BC, DE, HL, IX
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_level_medium_2_sam::
   ld hl, #gameState
   ld (hl), #game_playing      

   ld   hl, #_tilemap_medium_2_sam_end
   call man_game_change_tilemap

   ld   d, #1
   ld   e, #8
   call man_game_print_level
   
   CreateEntityXY player, bX*1, bY*3

   CreateEntityXY    enemy, bX*4  , bY*6
   ChangeAILimits      y, bY*6  , bY*10

   CreateEntityXY  ghost, bX*6  , bY*20
   ChangeAILimits      y, bY*18  , bY*21
   ChangeAILimits      x, bX*6 , bX*11

   CreateEntityXY    bat, bX*12+1   , bY*13
   ChangeAILimits      y, bY*12  , bY*16
   ChangeAILimits      x, bX*11   , bX*13

   CreateEntityXY babosa, bX*8  , bY*15
   ChangeAILimits x, bX*8  , bX*10
   CreateEntityXY babosa, bX*9  , bY*9
   ChangeAILimits x, bX*9  , bX*12

   ;; End entity
   CreateEntityXY end, bX*19+2 , bY*17
   call sys_physics_round_XY        ;; OJO: END lo necesita ya que esto solo se calcula en físicas
   call man_game_print_deaths
ret

.globl _tilemap_hard_1_sam_end

man_game_level_hard_1_sam::
   ld hl, #gameState
   ld (hl), #game_playing      

   ld   hl, #_tilemap_hard_1_sam_end
   call man_game_change_tilemap

   ld   d, #1
   ld   e, #9
   call man_game_print_level
   
   CreateEntityXY player, bX*18, bY*21
   call man_animations_facing_left_stopped

   CreateEntityXY    bat, bX*4   , bY*4+1
   ChangeAILimits      y, bY*4  , bY*8+1
   ChangeAILimits      x, bX*1   , bX*4+1

   CreateEntityXY    bat, bX*13   , bY*4+1
   ChangeAILimits      y, bY*4+1  , bY*7-1
   ChangeAILimits      x, bX*13   , bX*16-1
   ;; up-left
   CreateEntityXY    enemy, bX*6  , bY*10
   ChangeAILimits      y, bY*4  , bY*10
   ;; down-left
   CreateEntityXY    enemy, bX*10  , bY*17
   ChangeAILimits      y, bY*13  , bY*21
   ;; down-rigth
   CreateEntityXY    enemy, bX*13  , bY*21
   ChangeAILimits      y, bY*16  , bY*21

   ;; third entity   (AI)
   CreateEntityXY babosa, bX*7  , bY*22
   ChangeAILimits x, bX*3  , bX*7

   ;; End entity
   CreateEntityXY end, bX*19+2 , bY*11
   call sys_physics_round_XY        ;; OJO: END lo necesita ya que esto solo se calcula en físicas
   call man_game_print_deaths
ret


.globl _tilemap_hard_1_adrian_end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  AF, BC, DE, HL, IX
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_level_hard_1_adrian::

   ld hl,   #gameState
   ld (hl), #game_playing

   ld   hl, #_tilemap_hard_1_adrian_end
   call man_game_change_tilemap

   ld    e, #2                               ;; L = Foreground PEN (5)
   ld    d, #0                               ;; H = Background PEN (0)
   
   call cpct_setDrawCharM1_asm               ;; Set draw char colours

                                             ;; Calculate a video-memory location for printing a string
   ld    de, #memVideoPtr                    ;; DE = Pointer to start of the screen
   ld    b, #0                               ;; B = y coordinate
   ;ld    c, #3
   ld    c, #(screen_width/2-(string_final_level_size-1))   ;; C = x coordinate

   call  cpct_getScreenPtr_asm  

   ld    iy, #string_final_level              
   call  cpct_drawStringM1_asm   


   CreateEntityXY player, bX*5  , bY*5
   call man_animations_facing_left_stopped

   ;; Second entity   (AI)
   CreateEntityXY  ghost, bX*13  , bY*4+3
   ChangeAILimits      y, bY*4+3 , bY*6+3
   ChangeAILimits      x, bX*13  , bX*16

   ;; third entity   (AI)
   CreateEntityXY  enemy, bX*1   , bY*13
   ChangeAILimits      y, bY*7   , bY*20

   ;; fourth entity   (AI)
   CreateEntityXY  enemy, bX*2+1 , bY*10
   ChangeAILimits      y, bY*9   , bY*22

   ;; fifth entity   (AI)
   CreateEntityXY    bat, bX*9   , bY*11
   ChangeAILimits      y, bY*10  , bY*13-1
   ChangeAILimits      x, bX*9   , bX*12

   ;; sixth entity   (AI)
   CreateEntityXY    bat, bX*13  , bY*12+4
   ChangeAILimits      y, bY*12  , bY*15-1
   ChangeAILimits      x, bX*13  , bX*16

   ;; seventh entity   (AI)
   CreateEntityXY babosa, bX*6   , bY*18
   ChangeAILimits      x, bX*6   , bX*8+1

   ;; End entity
   CreateEntityXY    end, bX*9   , bY*5           
   call sys_physics_round_XY  ;; OJO: END lo necesita ya que esto solo se calcula en físicas
   call man_game_print_deaths
ret


.globl _tilemap_easy_1_adrian_end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  AF, BC, DE, HL, IX
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_level_easy_1_adrian::

   ld hl,   #gameState
   ld (hl), #game_playing

   ld   hl, #_tilemap_easy_1_adrian_end
   call man_game_change_tilemap

   ld   d, #0
   ld   e, #2
   call man_game_print_level

   CreateEntityXY player, bX*1+1  , bY*16

   ;; Second entity   (AI)
   CreateEntityXY babosa, bX*14   , bY*17
   ChangeAILimits      x, bX*13+2 , bX*16

   ;; End entity
   CreateEntityXY    end, bX*19+2 , bY*16           
   call sys_physics_round_XY  ;; OJO: END lo necesita ya que esto solo se calcula en físicas
   call man_game_print_deaths
ret


.globl _tilemap_easy_2_adrian_end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  AF, BC, DE, HL, IX
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_level_easy_2_adrian::

   ld hl,   #gameState
   ld (hl), #game_playing

   ld   hl, #_tilemap_easy_2_adrian_end
   call man_game_change_tilemap

   ld   d, #0
   ld   e, #3
   call man_game_print_level

   CreateEntityXY player, bX*1+1  , bY*11

   ;; Second entity   (AI)
   CreateEntityXY babosa, bX*15   , bY*12
   ChangeAILimits      x, bX*15 , bX*17+2

   ;; End entity
   CreateEntityXY    end, bX*19+2 , bY*11           
   call sys_physics_round_XY  ;; OJO: END lo necesita ya que esto solo se calcula en físicas
   call man_game_print_deaths
ret

.globl _tilemap_easy_3_adrian_end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  AF, BC, DE, HL, IX
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_level_easy_3_adrian::

   ld hl,   #gameState
   ld (hl), #game_playing

   ld   hl, #_tilemap_easy_3_adrian_end
   call man_game_change_tilemap

   ld   d, #0
   ld   e, #4
   call man_game_print_level

   CreateEntityXY player, bX*10  , bY*9
   call man_animations_facing_left_stopped

   ;; Second entity   (AI)
   CreateEntityXY babosa, bX*11   , bY*22
   ChangeAILimits      x, bX*11   , bX*14+1

   ;; End entity
   CreateEntityXY    end, bX*19+2 , bY*18

   call sys_physics_round_XY  ;; OJO: END lo necesita ya que esto solo se calcula en físicas
   call man_game_print_deaths
  
ret

.globl _easterEgg_end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Carga el easter egg
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  AF, BC, DE, HL, IX
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_level_easterEgg::
   ld hl, #gameState
   ld (hl), #game_playing      

   ld    a, #2
   ld    (onEasterEgg), a

   ld   hl, #_easterEgg_end
   call man_game_change_tilemap

   ld    e, #2                               ;; L = Foreground PEN (5)
   ld    d, #0                               ;; H = Background PEN (0)
   
   call cpct_setDrawCharM1_asm               ;; Set draw char colours

                                             ;; Calculate a video-memory location for printing a string
   ld    de, #memVideoPtr                    ;; DE = Pointer to start of the screen
   ld    b, #12                               ;; B = y coordinate
   ld    c, #3

   call  cpct_getScreenPtr_asm  

   ld    iy, #string_easter_egg              
   call  cpct_drawStringM1_asm    


   
   CreateEntityXY player, screen_width-2-spr_player_w, bY*6
   call man_animations_facing_left_stopped

   ;; End entity
   CreateEntityXY end, bX*19+3, bY*6
   ld    e_w(ix), #1
   call sys_physics_round_XY        ;; OJO: END lo necesita ya que esto solo se calcula en físicas
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Pinta por pantalla el nivel en el que estamos
;; ENTRADAS
;;  DE -> Number to be printed (D: Tens, 
;; SALIDAS
;;  -
;; MODIFICA
;;  AF, BC, DE, HL, IX
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_print_level:
   push de
   ld    e, #2                               ;; L = Foreground PEN (5)
   ld    d, #0                               ;; H = Background PEN (0)
   
   call cpct_setDrawCharM1_asm               ;; Set draw char colours

                                             ;; Calculate a video-memory location for printing a string
   ld    de, #memVideoPtr                    ;; DE = Pointer to start of the screen
   ld    b, #0                               ;; B = y coordinate (24 = 0x18)
   ld    c, #32                               ;; C = x coordinate (16 = 0x10)

   call  cpct_getScreenPtr_asm  

   ld    iy, #string_level                   ;;Loads "Level"
   call  cpct_drawStringM1_asm    

   pop de                     ;; d contains tens      e contains units
   ld a, d      
   sub #1
   jp  m, #setUnits
   setTens:
      push de
      ld      iy, #string_num
      ld      a, #string_numm
      add     d     
      ld      (iy), a
      ld    de, #memVideoPtr                     
      ld    b, #0                               
      ld    c, #44                              
      call  cpct_getScreenPtr_asm
      call  cpct_drawStringM1_asm
      pop de
   setUnits:
      ld      iy, #string_num
      ld      a, #string_numm
      add     e    
      ld      (iy), a
      ld    de, #memVideoPtr                    ;; DE = Pointer to start of the screen
      ld    b, #0                               ;; B = y coordinate (24 = 0x18)
      ld    c, #46                              ;; C = x coordinate (16 = 0x10)
      call  cpct_getScreenPtr_asm
      call  cpct_drawStringM1_asm

ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Programa principal que controla la partida
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  AF, BC, DE, HL, IX
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_play::
                                       ;; Cargar Menu principal
                                       ;; 2 opcoines
   man_game_play_main_loop:
      ld de, #man_game_play_loop
      push de

      ;; call (onEasterEgg ?  man_game_level_easterEgg : man_game_level_X)
      ld    hl, #man_game_level_easterEgg ;; Carga nivel easter egg
      ld    a, (#onEasterEgg)
      cp    #1
      call  nz, man_game_set_level          ;; Carga el nivel correspondiente (tanto si se reinicia como si se pasa al siguiete)
      jp (hl)
      
      
      
      call  sys_render_update
      man_game_play_loop:
         ;call  man_game_print_deaths
         call  sys_input_update         ;; Se llama al sistema de control para actualizar
         call  sys_ai_control_update
         call  sys_physics_update
         call  sys_animation_update    ;; ----------------------------------------------------
         call  sys_collisions_update   
         call  cpct_waitVSYNC_asm
         call  sys_render_update       ;; OJOOOO: hay animaciones sin renderizar la primera vez, copiar antes del bucle  
         call  man_entity_update
         
         ;ld    d, #2
         ;call sys_render_wait
                                        ;; Comprobar si el estado del juego es 0
         call  man_game_check_state     ;; esta linea se podria ahorrar
         sub   #game_playing                       ;; Si no se esta jugando   
         jr nz, #man_game_manage_level  ;; Jugador muere, se resetea el nivel
         
         jr man_game_play_loop

      man_game_manage_level:
         ;call  man_game_print_deaths
         jr man_game_play_main_loop

ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Pone en el puntero HL el nivel que se ha de jugar 
;; ENTRADAS
;;  -
;; SALIDAS
;;  HL: Puntero al nivel que se ha de jugar 
;; MODIFICA
;;  A, HL, IX
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_set_level::
   
   ld    a, (#onEasterEgg)
   or    a
   jr    nz, man_game_set_level_music_continue
      call man_music_enable_music
  man_game_set_level_music_continue:
   ld  a, (#levelCounter) 
   add a                    ;; Duplica porque las pos son de 2 en dos
   ld  d, #0x00
   ld  e, a
   ld  ix, #m_game_levels   ;; Apuntamos al principio de los niveles
   
   add ix, de
   ld  l, (ix)           ;; HL = m_game_leveles[levelCounter]
   inc de
   ld  h, 1(ix)
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Comprueba el estado de la partida. Deja en HL la rutina a
;; llamar para seguir el ciclo del juego 
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  A, HL
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_check_state:
   ld a, (#gameState)
ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Pasa al siguiente nivel
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  AF, BC, DE, HL
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_next_level:
   call man_game_reset_level
   call man_entity_update
   call  sys_render_update 
   ;call  cpct_waitVSYNC_asm
   
   ld hl, #gameState
   ld (hl), #game_next

   ;; if(onEasterEgg) return
   ld a, (#onEasterEgg)
   or a
   ret nz

   call man_music_enable_music
   ld de, #_song_next
   call man_music_init_music_25      

   ld hl, #levelCounter
   inc (hl)

   ld     hl, #_screennextlevel_z_end
   ld    de, #0xFFFF

   call cpct_zx7b_decrunch_s_asm

   ;call sys_render_update

   ld d, #pause_next_level                  
   call sys_render_wait
   
 ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Pasa al nivel easter egg
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  AF, BC, DE, HL
;; PRECONDICIONES
;;    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_easter_egg_level::
   ld hl, #gameState
   ld (hl), #game_next

   ld    a, #1
   ld   (#onEasterEgg), a

   call man_game_reset_level
   call man_entity_update
   call  sys_render_update 
   ;call  cpct_waitVSYNC_asm

   
 ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Resetea el nivel cuando el jugador muere
;; ENTRADAS
;;  IX - Apunta al jugador
;; SALIDAS
;;  -
;; MODIFICA
;;  AF, BC, DE, HL
;; PRECONDICIONES
;;  Debe haber espacio para crear una entidad  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_reset_level:
   call man_entity_set4destruction_all_entities
ret

man_game_add_deaths::
   ld hl, #deathcounterUnits
   inc (hl)
   ld a, (hl)
   sub #10
   jp p,  #addTen
   jp m, #continue 


   addTen:
      ld hl, #deathcounterUnits
      ld (hl), #0
      ld hl, #deathcounterTens
      inc (hl)
      ld a, (hl)
      sub #10
      jp p,  #addHundred
      jp m, #continue

   addHundred:
      ld hl, #deathcounterTens
      ld (hl), #0
      ld hl, #deathcounterUnits
      ld (hl), #0
      ld hl, #deathcounterHundreds
      inc (hl)
   continue:

ret

man_game_print_total_deaths::

   ld    de, #memVideoPtr                    ;; DE = Pointer to start of the screen
   ld    b, #130                               ;; B = y coordinate (24 = 0x18)
   ld    c, #10                               ;; C = x coordinate (16 = 0x10)

   call  cpct_getScreenPtr_asm  

   ld    iy, #string_total_deaths                   ;;Loads "deaths"
   call  cpct_drawStringM1_asm   
      
   ;;;;;;;;;;;;;;;;;;;;;;;; units;;;;
      ld      iy, #string_num
      ld      hl, #deathcounterUnits
      ld      a, (hl)
      add     #string_numm
         
      ld      (iy), a
      
      ld    de, #memVideoPtr                    ;; DE = Pointer to start of the screen
      ld    b, #130                               ;; B = y coordinate (24 = 0x18)
      ld    c, #60                             ;; C = x coordinate (16 = 0x10)
      call  cpct_getScreenPtr_asm
      call  cpct_drawStringM1_asm
   ;;;;;;;;;;;;;;;;;;;;;
      ld      iy, #string_num
      ld      hl, #deathcounterTens
      ld      a, (hl)
      add     #string_numm
         
      ld      (iy), a
      
      ld    de, #memVideoPtr                    ;; DE = Pointer to start of the screen
      ld    b, #130                               ;; B = y coordinate (24 = 0x18)
      ld    c, #58                              ;; C = x coordinate (16 = 0x10)
      call  cpct_getScreenPtr_asm
      call  cpct_drawStringM1_asm

   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   ;;;;;;;;;;;;;;;;;;;;;;;; units;;;;
      ld      iy, #string_num
      ld      hl, #deathcounterHundreds
      ld      a, (hl)
      add     #string_numm
         
      ld      (iy), a
      
      ld    de, #memVideoPtr                    ;; DE = Pointer to start of the screen
      ld    b, #130                               ;; B = y coordinate (24 = 0x18)
      ld    c, #56                             ;; C = x coordinate (16 = 0x10)
      call  cpct_getScreenPtr_asm
      call  cpct_drawStringM1_asm



ret


man_game_print_deaths::
   ld    de, #memVideoPtr                    ;; DE = Pointer to start of the screen
   ld    b, #0                               ;; B = y coordinate (24 = 0x18)
   ld    c, #57                               ;; C = x coordinate (16 = 0x10)

   call  cpct_getScreenPtr_asm  

   ld    iy, #string_deaths                   ;;Loads "deaths"
   call  cpct_drawStringM1_asm   
   print_deaths:
      ld    de, #memVideoPtr                    ;; DE = Pointer to start of the screen
      ;ld    b, #0                               ;; B = y coordinate (24 = 0x18)
      ;ld    c, #65                               ;; C = x coordinate (16 = 0x10)

      ;ld    de, #memVideoPtr                    ;; DE = Pointer to start of the screen
      ld    b, #0                               ;; B = y coordinate (24 = 0x18)
      ld    c, #2

   ;;;;;;;;;;;;;;;;;;;;;;;; units;;;;
      ld      iy, #string_num
      ld      hl, #deathcounterUnits
      ld      a, (hl)
      add     #string_numm
         
      ld      (iy), a
      
      ld    de, #memVideoPtr                    ;; DE = Pointer to start of the screen
      ld    b, #0                               ;; B = y coordinate (24 = 0x18)
      ld    c, #76                             ;; C = x coordinate (16 = 0x10)
      call  cpct_getScreenPtr_asm
      call  cpct_drawStringM1_asm
   ;;;;;;;;;;;;;;;;;;;;;
      ld      iy, #string_num
      ld      hl, #deathcounterTens
      ld      a, (hl)
      add     #string_numm
         
      ld      (iy), a
      
      ld    de, #memVideoPtr                    ;; DE = Pointer to start of the screen
      ld    b, #0                               ;; B = y coordinate (24 = 0x18)
      ld    c, #74                              ;; C = x coordinate (16 = 0x10)
      call  cpct_getScreenPtr_asm
      call  cpct_drawStringM1_asm

   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   ;;;;;;;;;;;;;;;;;;;;;;;; units;;;;
      ld      iy, #string_num
      ld      hl, #deathcounterHundreds
      ld      a, (hl)
      add     #string_numm
         
      ld      (iy), a
      
      ld    de, #memVideoPtr                    ;; DE = Pointer to start of the screen
      ld    b, #0                               ;; B = y coordinate (24 = 0x18)
      ld    c, #72                             ;; C = x coordinate (16 = 0x10)
      call  cpct_getScreenPtr_asm
      call  cpct_drawStringM1_asm

ret

man_game_end_level::
   ;;
   call man_game_add_deaths
   call man_music_enable_music
   ld de, #_song_death
   call man_music_init_music_50

   ld hl,   #gameState
   ld (hl), #game_stopped
   call     man_entity_disable_cmps_movable      ;; Quita componente movable de todas las entidades
   call     sys_animation_death                  ;; Asigna animacion de muerte a player
   call     sys_render_draw_death_animation      ;; Dibuja animacion

   call     man_game_reset_level
   
ret

;;======================================================================================
;; PRIVATE ROUTINES
;;======================================================================================

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Crea una entidad desde un template
;; ENTRADAS
;;  HL - Puntero a template de la entidad
;; SALIDAS
;;  IX - Puntero a la entidad
;; MODIFICA
;;  AF, BC, DE, HL
;; PRECONDICIONES
;;  Debe haber espacio para crear una entidad  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_create_template_entity::
   push  hl
   call  man_entity_create_one_entity  ;; Devuelve en DE la dirección de la entidad
   pop   hl
   ld    bc, #e_sizeof 
   ldir                                ;; Copia el player por defecto en la primera entidad
   
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Gestiona el salto de nuestro jugador principal
;; ENTRADAS
;;  IX - Puntero a la entidad jugador
;; SALIDAS
;;  
;; MODIFICA
;;  AF
;; PRECONDICIONES
;;    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_player_jump::
   ld  a, (#playerOnFloor)  ;; generalizar cambiando a atributo de ix
   cp #1
   ret nz

   call sys_physics_round_XY
   ;; Salta desde el suelo
   ld e_vy(ix), #jump_velocity
   ld a, #0
   ld (#playerOnFloor), a                        
                                          
ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Actualiza el estado del salto del jugador
;; ENTRADAS
;;  
;; SALIDAS
;;  -
;; MODIFICA
;;  AF
;; PRECONDICIONES
;;    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_floor_collision::
   ld     a, #1
   ld    (#playerOnFloor), a

   ld     a, e_cmps(ix)
   or    #e_cmps_input
   ld     e_cmps(ix), a
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Actualiza el estado del salto del jugador
;; ENTRADAS
;;  
;; SALIDAS
;;  -
;; MODIFICA
;;  AF
;; PRECONDICIONES
;;    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_player_on_air::
   ld  a, #0
   ld (#playerOnFloor), a
   ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Devuelve en DE el puntero al tilemap usado en el nivel actual
;; ENTRADAS
;;  
;; SALIDAS
;;  DE - puntero al tilemap
;; MODIFICA
;;  DE
;; PRECONDICIONES
;;    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_getTilemapPtrDE::
   ld de, #tilemapPtr
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Devuelve en HL el puntero al tileset usado en el nivel actual
;; ENTRADAS
;;  
;; SALIDAS
;;  HL - puntero al tilemap
;; MODIFICA
;;  DE
;; PRECONDICIONES
;;    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_getTilesetPtrHL::
   ld hl, #_tileset_00
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Devuelve en HL el puntero al tileset usado en el nivel actual
;; ENTRADAS
;;  
;; SALIDAS
;;  HL - puntero al tilemap final del tilemap
;; MODIFICA
;;  DE, AF, BC, IX, IY
;; PRECONDICIONES
;;    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_change_tilemap::
   ;; if(currentTilemap == newTilemap ) return
   ld   a, (currentTilemap)
   sub  l
   ld   b, a
   ld   a, (currentTilemap+1)
   sub  h
   or   b
   ret  z

   ;; currentTilemap = newTilemap
   ld (currentTilemap), hl

   ;; Descompresión del nuevo nivel
   ld    de, #tilemapPtrEnd
   call cpct_zx7b_decrunch_s_asm
   call sys_render_drawTilemap
ret


man_game_pause::

   ld    e, #3                              
   ld    d, #0                              
   
   call cpct_setDrawCharM1_asm               

   ld    de, #memVideoPtr                    
   ld    b, #0                               
   ld    c, #2

   

   call  cpct_getScreenPtr_asm  

   ld    iy, #string_pause                   ;;Loads "|| Paused"

   call  cpct_drawStringM1_asm

   call sys_input_wait_releaseKey  
  
   wait_pause:
      call sys_input_update_pause
      jr z, #wait_pause
      jr nz, #back_game  

   back_game:
      call sys_input_wait_releaseKey
                                   ;; C = x coordinate (16 = 0x10)

      ld    e, #2                               ;; L = Foreground PEN (5)
      ld    d, #0                               ;; H = Background PEN (0)
   
      call cpct_setDrawCharM1_asm               ;; Set draw char colours

      ld    de, #memVideoPtr                    ;; DE = Pointer to start of the screen
      ld    b, #0                               ;; B = y coordinate (24 = 0x18)
      ld    c, #2  

      call  cpct_getScreenPtr_asm  

      ld    iy, #string_clear_pause                 ;;Loads "Paused"

      call  cpct_drawStringM1_asm
      ret


     
   




ret