
;;	Copyright © CrackTime Studios - 2020
;;
;;	This file is part of Poisoned Escape
;;
;;    Poisoned Escape is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Poisoned Escape is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Poisoned Escape.  If not, see <https://www.gnu.org/licenses/>.
.include "menu.h.s"
.include "game.h.s"
.include "cpct_functions.h.s"
.include "../sys/physics.h.s"
.include "../sys/render.h.s"
.include "../sys/input.h.s"
.include "assets/screens/screenmenu_z.h.s"
.include "assets/screens/screencredits_z.h.s"
.include "assets/screens/screeninstructions_z.h.s"

;;======================================================================================
;; PUBLIC VARIABLES
;;======================================================================================


m_game_menu_options::
   .dw man_menu_loop
   .dw man_game_play                         ;; Press 1 -> Game Play
   .dw man_menu_load_instructions_screen     ;; Press 2 -> Game Instructions
   .dw man_menu_load_credits_screen          ;; Press 3 -> Game Credits
                                             ;; Press Q -> Return Game Menu

menuState:: .db 0                ;; 0 -> Menu Screen;    1-> Play;   2-> Instr;    3-> Credits


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Carga el menu del juego
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  -
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_menu_load_menu::
   call sys_input_wait_releaseKey
   ld    hl, #menuState
   ld    (hl), #0

   ld    hl, #_screenmenu_z_end
   ld    de, #0xFFFF

   call cpct_zx7b_decrunch_s_asm
   
   call sys_render_update
   
   man_menu_load_menu_main_loop:
      ld de, #man_menu_loop
      push de
      call man_game_set_menu_action  
      jp (hl)
      call sys_render_update

      man_menu_loop: 
         call  sys_input_update_menu
         ld    a, (#menuState)             ;; Cargamos el estado del menu
         or    a                           ;; si no es 0
         jr z, #man_menu_loop
      
      jr man_menu_load_menu_main_loop

ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Carga los creditos del juego
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  -
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_menu_load_credits_screen::
   call sys_input_wait_releaseKey

   ld    hl, #_screencredits_z_end
   ld     de, #0xFFFF

   call cpct_zx7b_decrunch_s_asm

   call sys_render_update

   man_load_credits_loop:
      call sys_input_update_menu_credits_and_instructions
      ld    a, (#menuState)             ;; Cargamos el estado del menu
      cp   #menu_credits               ;; Comprobamos si estamos en los credtos
      jr   z, #man_load_credits_loop
      
   jr man_menu_load_menu
ret




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Modifica el estado del menu 
;; ENTRADAS
;;  A: estado del menu que se quiere poner
;; SALIDAS
;;  
;; MODIFICA
;;  HL
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_menu_set_action::
   ld hl, #menuState
   ld (hl), a
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Carga en el estado del menu las instrucciones
;; ENTRADAS
;;  -
;; SALIDAS
;; 
;; MODIFICA
;;  
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_menu_load_instructions::
   ld a, #menu_instruct
   call man_menu_set_action   
ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Carga las instrucciones del juego
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  -
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_menu_load_instructions_screen::
   call sys_input_wait_releaseKey

   ld    hl, #_screeninstructions_z_end
   ld     de, #0xFFFF

   call cpct_zx7b_decrunch_s_asm

   call sys_render_update

   man_load_instructions_loop:
      call sys_input_update_menu_credits_and_instructions
      ld    a, (#menuState)             ;; Cargamos el estado del menu
      cp   #menu_instruct               ;; Comprobamos si estamos en los credtos
      jr   z, #man_load_instructions_loop
      
   jr man_menu_load_menu
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Carga en el estado el juego
;; ENTRADAS
;;  -
;; SALIDAS
;; 
;; MODIFICA
;;  
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_menu_load_game::
   ld a, #menu_play
   call man_menu_set_action   
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Carga en el estado del menu los creditos
;; ENTRADAS
;;  -
;; SALIDAS
;; 
;; MODIFICA
;;  
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

man_menu_load_credits::
   ld a, #menu_credits
   call  man_menu_set_action  
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Carga en el estado del menu el menu principal
;; ENTRADAS
;;  -
;; SALIDAS
;; 
;; MODIFICA
;;  
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_menu_load_main_menu::
   ld a, #menu_screen
   call  man_menu_set_action  
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Pone en el puntero HL la opcion del menu que se ha de cargar 
;; ENTRADAS
;;  -
;; SALIDAS
;;  HL: Puntero a la opcion de menu que se ha de poner
;; MODIFICA
;;  A, HL, IX
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_game_set_menu_action::
   ld  a, (#menuState) 
   add a                    ;; Duplica porque las pos son de 2 en dos
   ld  d, #0x00
   ld  e, a
   ld  ix, #m_game_menu_options   ;; Apuntamos al principio de los niveles
   
   add ix, de
   ld  l, (ix)           ;; HL = m_game_menu_options[gameState]
   inc de
   ld  h, 1(ix)
ret


