;;	Copyright © CrackTime Studios - 2020
;;
;;	This file is part of Poisoned Escape
;;
;;    Poisoned Escape is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Poisoned Escape is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Poisoned Escape.  If not, see <https://www.gnu.org/licenses/>.

.include "sys/ai_control.h.s"
.include "man/game.h.s"
;.include "cpct_functions.h.s"
.include "man/entity.h.s"
;.include "../sys/physics.h.s"
;.include "../sys/render.h.s"
;.include "../assets/assets.h.s"
.include "man/entity_templates.h.s"
;.include "sys/animation.h.s"
;.include "../sys/input.h.s"
;.include "../sys/collisions.h.s"
;.include "../assets/maps/tilemap1.h.s"

;;======================================================================================
;; PUBLIC VARIABLES
;;======================================================================================



;;======================================================================================
;; PRIVATE VARIABLES
;;======================================================================================



;;======================================================================================
;; PUBLIC ROUTINES
;;======================================================================================



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS AI CONTROL INIT
;;  Actualiza todas las entidades de IA
;; ENTRADAS
;;  IX = Puntero a la entidad del array
;;  A  = Elementos en el array
;; SALIDAS
;;  
;; MODIFICA
;;  AF, BC, DE, IX
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_ai_control_init::
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS AI CONTROL UPDATE
;;  Actualiza todas las entidades de IA
;; ENTRADAS
;;  IX = Puntero a la entidad del array
;;  A  = Elementos en el array
;; SALIDAS
;;  
;; MODIFICA
;;  AF, BC, DE, IX
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_ai_control_update::
    ld    hl, #sys_ai_control_update_one_entity
    ld     c, #e_cmps_ai                ;; Todas las que se controlen por IA
    call   man_entity_for_all_matching
ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Modifica las entidades que tienen IA
;; ENTRADAS
;;  - IX (entidad)
;; SALIDAS
;;  -
;; MODIFICA
;;  
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_ai_control_update_one_entity::
    ld a, e_ai_st(ix)
    cp #e_ai_st_noAI             ;; si son iguales pone z = 0
    jr z, _no_AI_ent

    _AI_entity: 
        ;; Comprobar si es stand by o move to
        call sys_ai_move_to

    _no_AI_ent:
ret

;sys_ai_stand_by::
    
 ;   ret


sys_ai_control_move_on_y_axis::

    ld  a, e_ai_aim_init_y(ix)
    sub  e_y_rounded(ix)             ;; Limite superior 
    jr nc, #move_downwards_ai       ;; Se encuentra en limite superior?
                             ;; 
    ld  a, e_y_rounded(ix)                  
    sub  e_ai_aim_end_y(ix)  ;; Limite inferior
    jr nc, #move_upwards_ai      ;; Se encuentra en limite inferior?

    ld a, e_vy(ix)           ;; ver si es positiva
    or a
    jp p, #move_downwards_ai ;; va hacia abajo

    move_upwards_ai:
        ld e_vy(ix), #-spider_velocity
    ret

    move_downwards_ai:          ;; yendo hacia abajo
        ld e_vy(ix), #spider_velocity
    ret

sys_ai_control_move_on_x_axis::
    ld  a, e_ai_aim_init_x(ix)
    sub  e_x_rounded(ix)                    
    jr nc, #move_rightwards_ai          ;; Se encuentra en limite derecho?

    ld  a, e_x_rounded(ix)                      
    sub  e_ai_aim_end_x(ix)     
    jr nc, #move_leftwards_ai           ;; Se encuentra en limite izquierdo?

    ld a, e_vx(ix)                      ;; ver si es positiva
    or a    
    jp p, #move_rightwards_ai           ;; va hacia abajo
    move_leftwards_ai:
        ld  de, #-babosa_velocity
        ld  e_vx(ix), d
        ld  e_vxf(ix), e

    ret
    move_rightwards_ai:                 ;; yendo hacia abajo
        ld  de, #babosa_velocity
        ld  e_vx(ix), d
        ld  e_vxf(ix), e
    ret

sys_ai_control_ghost_wave::
     
    ld  a, e_ai_aim_init_x(ix)
    sub  e_x_rounded(ix)                    
    jr nc, #move_rightwards_ghost_ai      ;; Se encuentra en limite derecho?
                              
    ld  a, e_x_rounded(ix)                  
    sub  e_ai_aim_end_x(ix) 
    jr nc, #move_leftwards_ghost_ai       ;; Se encuentra en limite izquierdo?

    ld a, e_vx(ix)                        ;; ver si es positiva
    or a
    jp p, #move_rightwards_ghost_ai       ;; va hacia abajo
    move_leftwards_ghost_ai:
        ld  de, #-ghost_velocity_x
        ld  e_vx(ix), d
        ld  e_vxf(ix), e
    jr move_on_y_axis_ghost
    ;ret
    move_rightwards_ghost_ai:             ;; yendo hacia abajo

        ld  de, #ghost_velocity_x
        ld  e_vx(ix), d
        ld  e_vxf(ix), e
    jr move_on_y_axis_ghost
    ;ret

    move_on_y_axis_ghost:
        ld  a, e_ai_aim_init_y(ix)
        sub  e_y_rounded(ix)             ;; Limite superior 
        jr nc, #move_downwards_ai_ghost  ;; Se encuentra en limite superior?
                                 
        ld  a, e_y_rounded(ix)                  
        sub  e_ai_aim_end_y(ix)          ;; Limite inferior
        jr nc, #move_upwards_ai_ghost    ;; Se encuentra en limite inferior?

        ld a, e_vy(ix)                   ;; ver si es positiva
        or a
        jp p, #move_downwards_ai_ghost   ;; va hacia abajo

        move_upwards_ai_ghost:
            ld  de, #-ghost_velocity_y
            ld  e_vy(ix), d
            ld  e_vyf(ix), e
        ret

        move_downwards_ai_ghost:          ;; yendo hacia abajo
             ld  de, #ghost_velocity_y
             ld  e_vy(ix), d
             ld  e_vyf(ix), e
        ret


ret


sys_ai_control_bat_wave::
    ;call sys_ai_control_move_on_x_axis
     ld  a, e_ai_aim_init_x(ix)
    sub  e_x_rounded(ix)                    
    jr nc, #move_rightwards_bat_ai      ;; Se encuentra en limite derecho?
                              
    ld  a, e_x_rounded(ix)                  
    sub  e_ai_aim_end_x(ix) 
    jr nc, #move_leftwards_bat_ai       ;; Se encuentra en limite izquierdo?

    ld a, e_vx(ix)                  ;; ver si es positiva
    or a
    jp p, #move_rightwards_bat_ai       ;; va hacia abajo
    move_leftwards_bat_ai:
        ld  de, #-bat_velocity_x
        ld  e_vx(ix), d
        ld  e_vxf(ix), e
    jr move_on_y_axis_bat
    ;ret
    move_rightwards_bat_ai:          ;; yendo hacia abajo

        ld  de, #bat_velocity_x
        ld  e_vx(ix), d
        ld  e_vxf(ix), e
    jr move_on_y_axis_bat
    ;ret

    move_on_y_axis_bat:
        ld  a, e_ai_aim_init_y(ix)
        sub  e_y_rounded(ix)             ;; Limite superior 
        jr nc, #move_downwards_ai_bat       ;; Se encuentra en limite superior?
                                ;; 
        ld  a, e_y_rounded(ix)                  
        sub  e_ai_aim_end_y(ix)  ;; Limite inferior
        jr nc, #move_upwards_ai_bat      ;; Se encuentra en limite inferior?

        ld a, e_vy(ix)           ;; ver si es positiva
        or a
        jp p, #move_downwards_ai_bat ;; va hacia abajo

        move_upwards_ai_bat:
            ld  de, #-bat_velocity_y
            ld  e_vy(ix), d
            ld  e_vyf(ix), e
        ret

        move_downwards_ai_bat:          ;; yendo hacia abajo
             ld  de, #bat_velocity_y
             ld  e_vy(ix), d
             ld  e_vyf(ix), e
        ret


ret

sys_ai_move_to::

    ld l, e_ai_movPtr_l(ix)
    ld h, e_ai_movPtr_h(ix)
    
    jp (hl)
    
    ret