;;	Copyright © CrackTime Studios - 2020
;;
;;	This file is part of Poisoned Escape
;;
;;    Poisoned Escape is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Poisoned Escape is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Poisoned Escape.  If not, see <https://www.gnu.org/licenses/>.

.include "animation.h.s" ;; ---------------------------------------------------------------------
.include "collisions.h.s"

.include "cpct_functions.h.s"
.include "cpctelera.h.s"
.include "../man/entity.h.s"
.include "../man/animations.h.s"
.include "../assets/assets.h.s"
.include "../man/entity_templates.h.s"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; descripcion
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  -
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_animation_init::
   ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Actualiza las físicas de todas las entidades que deban hacerlo
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  -
;; PRECONDICIONESld    
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_animation_update::
   ld    hl, #sys_animation_update_one_entity
   ld     c, #e_cmps_anime              ;; Todas las que tipo anime
   call  man_entity_for_all_matching
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Aplica la animacion a una entidad
;; ENTRADAS
;;  IX - puntero a la entidad 
;; SALIDAS
;;  -
;; MODIFICA
;;  
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_animation_update_one_entity::
    call checkCollisionWall         ;; Comprueba si hay que mostrar animacion de deslizarse en pared
    cp #0                           ;; Si A == 0, no se ha modificado (no hay colision)
    jr nz, _updateStatus
_normalUpdate:
    call calculateAnimStatus        ;; Comprueba si la animacion actual es la misma que antes, guarda la nueva en -> B
    ld d, #0        
    ld a, e_animSt(ix)              ;; Estado actual de la entidad
    cp b                            ;; Resta pero no modifica A IF (a == b)
    jr z, _continueAnim             ;; Salta si tienen el mismo estado
    call newAnimationStatus         ;; Comprueba si el estado esta al reves (left)
_updateStatus:
    call update_anim
_continueAnim:
    ld e, e_animPtr_l(ix)   ;; Puntero a posicion del vector, no sprite
    ld d, e_animPtr_h(ix)

    ex de, hl               ;; Hl apunta a dir del vector con dir del sprite
_return2Anim:               ;;                            E       CD        HL
    ld e, (hl)              ;; Array de animaciones:  [(frames, sprite), (frames, srpite), (nullptr, PTR_init)]
    inc hl

    ld d, (hl)  
    inc hl                  ;; HL apunta siguiente elem del vector

    ld a, e                 ;; Si DE -> null pointer  
    or d                    ;; reiniciar animacion  
    jr z, _resetAnim

    ld c, (hl)              ;; Direccion de memoria del sprite (1)

    ld e_sprPtr_l(ix), d    ;; guarda en la entidad   
    ld e_sprPtr_h(ix), c    ;; puntero a siguiente sprit

    ld a, e_animFPS(ix)     ;; franes por segundo
    dec a  
    or a  
    jr nz, _end             ;; Mientras no sea 0, no actualiza sprite

    ld a, e                 ;; Reinicio frames por sprite   
    inc hl
    ld e_animPtr_l(ix), l   ;; Guarda siguiente paso  
    ld e_animPtr_h(ix), h   ;; en la animacion

    jr _end
_resetAnim:    
    ld a, (hl)              ;; cargo en HL aquello a lo que apunta HL -> HL = (*HL)
    inc hl
    ld h, (hl)
    ld l,a
    jr _return2Anim
_end:
    ld e_animFPS(ix), a
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Comprueba si colisiona con pared, animacion de bajada
;; ENTRADAS
;;  IX - Puntero a entidad
;; SALIDAS
;;  B - Next Anim Status // A -> 0 (no colisiona)
;; MODIFICA
;;  A, B
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
checkCollisionWall:
    ld b, #0
    ld a, e_tile_collisions(ix)
    cp #collision_left
    jr z, _wallLeft
    cp #collision_right
    jr z, _wallRight
    ld a, #0
    jr _check_end
_wallLeft:
    set 0, b
    set 2, b
    set 3, b
    jr _check_end
_wallRight:
    set 1, b
    set 2, b
    set 3, b
_check_end:
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Establecer nuevo estado de animacion
;; ENTRADAS
;;  A - Estado anterior de la animacion
;; SALIDAS
;;  B - Next Anim Status
;; MODIFICA
;;  A, D
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
newAnimationStatus:         ;; antes de guardar la nueva animacion, necesitamos saber si hay que mostrar la animacion de izquierda
    set 0, d                ;; Comprobar anterior movimiento izquierda
    cp d
    jr z, _prevLeft
                   
    set 1, d                ;; Comprobar anterior es parado izquierda
    cp d
    jr z, _prevstopLeft
    
    set 3, d                ;; Comprobar anterior es arriba mirando izquierda
    cp d
    jr z, _prevupLeft
      
    res 3, d                ;; Comprobar anterior es abajo mirando izquierda
    set 2, d
    cp d
    jr z, _prevdownLeft

    ld d, #0
    set 0, d
    set 3, d
    cp d
    jr z, _prevUp_Left

    res 3, d
    set 2, d
    cp d
    jr z, _prevDown_Left

    set 3, d
    cp d
    jr z, _prevWallLeft

    ret

_prevWallLeft:
    ld a, #0
    set 2, a
    cp b
    jr z, _currentDownLeft
    ret

_prevDown_Left:
    ld a, #0
    set 2, a
    cp b
    jr z, _currentDownLeft
    ret
_prevUp_Left:
    ld a, #0
    set 3, a
    cp b
    jr z, _currentUpLeft
    res 3, a
    cp b
    jr z, _currentStopLeft
    ret
_prevLeft:
    ld a, #0
    cp b
    jr z, _currentStopLeft
    ret
_currentStopLeft:
    ld b, #0
    set 0, b
    set 1, b
    ret
_prevstopLeft:
    ld a, #0
    set 3, a
    cp b
    jr z, _currentUpLeft
    ld a, #0
    set 2, a
    cp b
    jr z, _currentDownLeft
    ret
_currentUpLeft:
    ld b, #0
    set 0, b
    set 1, b
    set 3, b
    ret
_prevupLeft:
    ld a, #0
    cp b
    jr z, _currentStopLeft
    set 2, a
    cp b
    jr z, _currentDownLeft
    ret
_currentDownLeft:
    ld b, #0
    set 0, b
    set 1, b
    set 2, b
    ret
_prevdownLeft:
    ld a, #0
    cp b
    jr z, _currentStopLeft
    set 0, a
    set 2, a
    cp b
    jr z, _currentLeftDown

_currentLeftDown:
    ld b, #0
    set 0, b
    set 2, b
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Asigna animacion de meurte a player
;; ENTRADAS
;;  IX - puntero a la entidad 
;; SALIDAS
;;  HL -> siguiente pos vector animcacion (siguiente sprite)
;;  DE -> sprite actual
;; MODIFICA
;;  B, DE, HL
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_animation_death::
   ld b, #0                               ;; Asigna animacion de muerte a player (suponemos que esta en IX)
   set 2, b
   set 3, b

   call update_anim                 

   ld e, (hl)
   inc hl
   ld d, (hl)
   inc hl

   ld e_sprPtr_l(ix), e                   ;; guarda en la entidad   
   ld e_sprPtr_h(ix), d                   ;; puntero a siguiente sprite

   ld e_animPtr_l(ix), l                  ;; Guarda siguiente paso
   ld e_animPtr_h(ix), h                  ;; en la animacion
   ret

update_anim:
   ld e_animSt(ix), b                     ;; Actualizado estado de animacion al nuevo
   ld c, e_animTable(ix)         
   call man_animations_getAnimHL          ;; Devuelve siguiente animacion (right/stop/up-right, etc...)
   ld e_animPtr_l(ix), l                  ;; Guarda siguiente paso  -> devuelve primera parte de la siguiente animacion: frames
   ld e_animPtr_h(ix), h                  ;; en la animacion
   
   ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Calcula el estado de la animcacion
;; ENTRADAS
;;  IX - puntero a la entidad 
;; SALIDAS
;;  B - Anim Status
;; MODIFICA
;;  AF
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; UDRL
;; 0000 - Stop
;; 0001 - left
;; 0010 - right
;; 0100 - down
;; 1000 - up
calculateAnimStatus::
    ld d, #0
    set 0, d

    ld b, #0                ;; Estado base
    ld a, e_vx(ix)          ;; a = e_vx
    or e_vxf(ix)
    jr z, _vxzero           ;; si es 0
_vxnozero:
    ;;jp m, _vxnegative
    ld a, e_vx(ix)
    and #0x80
    jr nz, _vxnegative
_vxpositive:
    set 1, b                ;; 7654 3210
    jr _vxzero
_vxnegative:
    set 0, b 
_vxzero:
    ld a, e_vy(ix)          ;; a = e_vy
    or e_vyf(ix)
    jr z, _vyzero           ;; si es 0
_vynozero:
    ld a, e_vy(ix)
    and #0x80
    jr nz, _vynegative
_vypositive:
    set 2, b                ;; 7654 3210
    jr _vyzero
_vynegative:
    set 3, b 
_vyzero:
    ld a, #0
    cp b
    jr z, _currentZero

    set 2, a
    cp b
    jr z, _currentDown

    res 2, a
    set 3, a
    cp b
    jr z, _currentUp
    jr _fin
_currentZero:
    ld a, e_animSt(ix)
    set 1, d
    cp d
    jr z, _stopLeft
    jr _fin
_stopLeft:
    ld b, #0
    set 0, b
    set 1, b
    jr _fin
_currentDown:
    ld a, e_animSt(ix)
    set 1, d
    set 2, d
    cp d
    jr z, _downLeft
    jr _fin
_downLeft:
    ld b, #0
    set 0, b
    set 1, b
    set 2, b
    jr _fin
_currentUp:
    ld a, e_animSt(ix)
    set 1, d
    set 3, d
    cp d
    jr z, _upLeft
    set 0, d
    cp d
    jr z, _upLeft
    jr _fin
_upLeft:
    ld b, #0
    set 0, b
    set 1, b
    set 3, b
_fin:
    ret
;;======================================================================================
;; PRIVATE ROUTINES
;;======================================================================================

