;;	Copyright © CrackTime Studios - 2020
;;
;;	This file is part of Poisoned Escape
;;
;;    Poisoned Escape is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Poisoned Escape is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Poisoned Escape.  If not, see <https://www.gnu.org/licenses/>.

.include "collisions.h.s"
.include "cpctelera.h.s"
.include "cpct_functions.h.s"
.include "../man/entity.h.s"
.include "../man/game.h.s"
.include "../man/entity_templates.h.s"
.include "physics.h.s"


;;======================================================================================
;; PUBLIC VARIABLES
;;======================================================================================




;;======================================================================================
;; PRIVATE VARIABLES
;;======================================================================================



;;======================================================================================
;; PUBLIC ROUTINES
;;======================================================================================

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Inicializa el sistema de colisiones
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  -
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_collisions_init::
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Comprueba las colisionesde todas las posibles entidades
;; colisionables
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  AF. DE, BC, HL, IX, IY
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_collisions_update::
   ld    hl, #sys_collisions_check
   ld     c, #e_cmps_collisionable
   call  man_entity_for_all_pairs_matching
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Comprueba si dos entidades colisionan
;; ENTRADAS
;;  IX - Dirección de la primera entidad
;;  IY - Dirección de la segunda entidad
;; SALIDAS
;;  -
;; MODIFICA
;;  TO DO
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_collisions_check::
   ld a, e_collides_against(ix)              ;; ix puede colisionar con iy?
   and e_type(iy)
   jr  nz, sys_collisions_check_bounding_boxes

   ld a, e_collides_against(iy)              ;; iy puede colisionar con ix?
   and e_type(ix)
   ret z                                     ;; No pueden colisionar

  sys_collisions_check_bounding_boxes:       ;; Pueden colisionar
   ;; if( e_x_rounded(ix) + e_w(ix) - e_x_rounded(iy) < 0 ) => no hay colisión
   ld    a, e_x_rounded(ix)
   add   e_w(ix)
   dec   a
   sub   e_x_rounded(iy)                                   
   ret   c                                   

   ;; if( e_x_rounded(iy) + e_w(iy) - e_x_rounded(ix) < 0 ) => no hay colisión
   ld    a, e_x_rounded(iy)
   add   e_w(iy)
   dec   a
   sub   e_x_rounded(ix)                                   
   ret   c                                   
   
   ;; if( e_y_rounded(ix) + e_h(ix) - e_y_rounded(iy) < 0 ) => no hay colisión
   ld    a, e_y_rounded(ix)
   add   e_h(ix)
   dec   a
   sub   e_y_rounded(iy)                                   
   ret   c                                   

   ;; if( e_x_rounded(iy) + e_h(iy) - e_x_rounded(ix) < 0 ) => no hay colisión
   ld    a, e_y_rounded(iy)
   add   e_h(iy)
   dec   a
   sub   e_y_rounded(ix)                                   
   ret   c  

  sys_collisions_check_both_collided:

   ld    a, e_type(ix)
   or   e_type(iy)

   cp   #e_type_player|e_type_enemy
   call z, #man_game_end_level

   cp   #e_type_player|e_type_end
   call z, #man_game_next_level

   cp   #e_type_player|e_type_easter_egg_end
   call z, #man_game_easter_egg_level
  
   ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Comprueba si la entidad colisiona con tile
;; ENTRADAS
;;  TO DO
;; SALIDAS
;;  -
;; MODIFICA
;;  TO DO
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_collisions_checkAndTreat_tile_collisionX::
   ld     a, e_vx(ix)
   cp    #0
   jp     p, _positive_vx
   jp     m, _negative_vx
   ret
  _negative_vx:
   ld    b, e_y_rounded(ix)
   ld    c, e_x_rounded(ix)
   call  sys_collisions_getPtrInsideTilemapHL

   ld     d, #0      ;; Máscara de direcciones collisionadas -> ¿Generalizar y llevar a estado?
   ld    bc, #20     ;; plataformas de mínimo 8 de altura y 20 tiles de ancho de tilemap

   call check_left_collision     ;; colisión esquina superior izquierda (0px,0px)
   ;jp     z, collision_produced_X

   add   hl, bc
   call check_left_collision     
   ;jp     z, collision_produced_X

   ld    a, e_y_rounded(ix)
   add   #15
   ld    b, a
   ld    c, e_x_rounded(ix) 

   ld    a, d
   ld    (#save_prev_collisionsX1), a
   call  sys_collisions_getPtrInsideTilemapHL
  save_prev_collisionsX1 == . + 1
   ld     d, #0
   
   call check_left_collision     ;; colisión centro izquierda (0px,15px)
   ld    a, d
   and   #0xFF
   jr    nz, collision_produced_X

   
   ret
  _positive_vx:
   ld     b, e_y_rounded(ix)
   ld     a, e_x_rounded(ix)
   add   #2
   ld     c, a
   call  sys_collisions_getPtrInsideTilemapHL

   ld     d, #0      ;; Máscara de direcciones collisionadas -> ¿Generalizar y llevar a estado?
   ld    bc, #20     ;; plataformas de mínimo 8 de altura y 20 tiles de ancho de tilemap

   call check_right_collision     ;; colisión esquina superior izquierda (0px,0px)
   ;jp     z, collision_produced_X

   add   hl, bc
   call check_right_collision     
   ;jp     z, collision_produced_X
   
   ld     a, e_x_rounded(ix)
   add   #2
   ld     c, a
   ld    a, e_y_rounded(ix) 
   add   #15
   ld    b, a
   
   ld    a, d
   ld    (#save_prev_collisionsX2), a
   call  sys_collisions_getPtrInsideTilemapHL
  save_prev_collisionsX2 == . + 1
   ld     d, #0

   call check_right_collision     ;; colisión centro izquierda (0px,15px)
   ld    a, d
   and   #0xFF
   jr    nz, collision_produced_X
   ret
  collision_produced_X:
   ld    a, e_tile_collisions(ix)
   or    d
   ld    e_tile_collisions(ix), a
   call sys_physics_collision_behaviour
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Comprueba si la entidad colisiona con tile
;; ENTRADAS
;;  TO DO
;; SALIDAS
;;  -
;; MODIFICA
;;  TO DO
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_collisions_checkAndTreat_tile_collisionY::
   ld     a, e_vy(ix)
   cp    #0
   jp    p, #_positive_vy
   jp    m, #_negative_vy
   ret

  _positive_vy:
   ld    a, e_y_rounded(ix)
   ld    c, e_x_rounded(ix) 
   add   #15
   ld    b, a
   call  sys_collisions_getPtrInsideTilemapHL

   ld     d, #0                  ;; Máscara de direcciones collisionadas

   call check_down_collision     ;; colisión esquina inferior izquierda
   jp    z, collision_produced_down1
      ld     a, e_x_rounded(ix)
      add   #2
      ld     c, a
      ld     a, e_y_rounded(ix)
      add   #15
      ld     b, a

      ld    a, d
      ld    (#save_prev_collisionsLeftY1), a
      call  sys_collisions_getPtrInsideTilemapHL
     save_prev_collisionsLeftY1 == . + 1
      ld     d, #0
      call check_enemy_tile_collision
  collision_produced_down1:
  ;;;;;; esquina inferior derecha vv
   ld     a, e_x_rounded(ix)
   add   #2
   ld     c, a
   ld     a, e_y_rounded(ix)
   add   #15
   ld     b, a

   ld    a, d
   ld    (#save_prev_collisionsY1), a
   call  sys_collisions_getPtrInsideTilemapHL
  save_prev_collisionsY1 == . + 1
   ld     d, #0

   call check_down_collision     ;; colisión esquina inferior izquierda
   jp    z, collision_produced_down2
      ld    a, e_y_rounded(ix)
      ld    c, e_x_rounded(ix) 
      add   #15
      ld    b, a

      ld    a, d
      ld    (#save_prev_collisionsRightY1), a
      call  sys_collisions_getPtrInsideTilemapHL
     save_prev_collisionsRightY1 == . + 1
      ld     d, #0
      call check_enemy_tile_collision
  collision_produced_down2:

   ;;;;;;;; colisión en el centro para tiles (la estela no choca y tampoco con la parte rocosa del lago tóxico)
   ld     a, e_x_rounded(ix)
   add   #1
   ld     c, a
   ld     a, e_y_rounded(ix)
   add   #15
   ld     b, a

   ld    a, d
   ld    (#save_prev_collisionsY2), a
   call  sys_collisions_getPtrInsideTilemapHL
  save_prev_collisionsY2 == . + 1
   ld     d, #0

   call check_enemy_tile_collision

   ld    a, d
   and   #0xFF
   jr    nz, collision_produced_Y

   call man_game_player_on_air

   ret
  _negative_vy:
   ;;;;;;;;;;
   ;;;;;;;;;;
   ld    b, e_y_rounded(ix)
   ld    c, e_x_rounded(ix) 

   call  sys_collisions_getPtrInsideTilemapHL

   ld     d, #0                  
   
   call check_up_collision       ;; colisión esquina superior izquierda 
   ;jr    z, collision_produced_Y

  ;;;;;; esquina superior derecha vv
   ld     a, e_x_rounded(ix)
   add   #2
   ld     c, a
   ld     b, e_y_rounded(ix)
   
   ld    a, d
   ld    (#save_prev_collisionsY3), a
   call  sys_collisions_getPtrInsideTilemapHL
  save_prev_collisionsY3 == . + 1
   ld    d, #0

   call check_up_collision       ;; colisión esquina superior derecha (px,8-15px)
   
   ld    a, d
   and   #0xFF
   jr    nz, collision_produced_Y
   ret
   ;;;;;;;;;;
   ;;;;;;;;;;


  collision_produced_Y:
   ld    a, e_tile_collisions(ix)
   or    d
   ld    e_tile_collisions(ix), a
   call sys_physics_collision_behaviour
   ret


;;======================================================================================
;; PRIVATE ROUTINES
;;======================================================================================

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Comprueba si la entidad colisiona con tile por la parte 
;; izquierda de la entidad
;; ENTRADAS
;;  HL - Puntero al tile
;; SALIDAS
;;  D - Añade bit de colisión izquierda si se produce 
;;  Z - 1 si hay colisión
;; MODIFICA
;;  AF, D
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
check_left_collision:
   call check_enemy_tile_collision
   ld     a, (hl)
   cp    #first_collisionable_tile_pos
   jr    c, #check_left_collision_end          ;; z=1 si es tile no colisionable (fondo)
   
   ;; Hay colisión
   ld     a, e_x_rounded(ix)
   and   #0b0011
   sub   #0b0011
   jr    nz, check_left_collision_end
      ;; Colisiona el borde
      set  collision_left_bit, d
      and   #0    ;; Para mantener la congruencia en la salida z
      ret
  check_left_collision_end:
   or #1
   ret
   
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Comprueba si la entidad colisiona con tile
;; ENTRADAS
;;  HL - Puntero al tile
;; SALIDAS
;;  D - Añade bit de colisión izquierda si se produce 
;;  Z - 1 si hay colisión
;; MODIFICA
;;  AF, D
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
check_right_collision:
   call check_enemy_tile_collision
   ld     a, (hl)
   cp    #first_collisionable_tile_pos
   jr    c, #check_right_collision_end          ;; z=1 si es tile no colisionable (fondo)
   
   ;; Hay colisión
   ld     a, e_x_rounded(ix)
   and   #0b0011
   sub   #0b0010
   jr    nz, check_right_collision_end
      ;; Colisiona el borde
      set  collision_right_bit, d
      and    #0               ; z=1
      ret
  check_right_collision_end:
   or   #1                    ; z=0
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Comprueba si la entidad colisiona con tile en el borde
;; superior del tile; inferior de la entidad de la entidad
;; ENTRADAS
;;  HL - Puntero al tile
;; SALIDAS
;;  D - Añade bit de colisión inferior si se produce 
;;  Z - 1 si hay colisión
;; MODIFICA
;;  AF, D
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
check_down_collision:      ;; se puede generalizar
   ;call check_enemy_tile_collision
   ld     a, (hl)
   cp    #first_collisionable_tile_pos
   jr    c, check_down_collision_end
   ;; Hay colisión
      set  collision_down_bit, d
      and  #0              ; z=1
      ret
  check_down_collision_end:
   or  #1                  ; z=0
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Comprueba si la entidad colisiona con tile en el borde
;; sinferior del tile; superior de la entidad de la entidad
;; ENTRADAS
;;  HL - Puntero al tile
;; SALIDAS
;;  D - Añade bit de colisión inferior si se produce 
;;  Z - 1 si hay colisión
;; MODIFICA
;;  AF, D
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
check_up_collision:      ;; se puede generalizar
   call check_enemy_tile_collision
   ld     a, (hl)
   cp    #first_collisionable_tile_pos
   jr    c, check_up_collision_end
   ;; Hay colisión
      set  collision_up_bit, d
      and  #0              ; z=1
      ret
  check_up_collision_end:
   or  #1                  ; z=0
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Comprueba si la entidad colisiona con un tile enemigo
;; ENTRADAS
;;  HL - Puntero al tile
;; SALIDAS
;;  D - Añade bit de colisión inferior si se produce 
;;  Z - 1 si hay colisión
;; MODIFICA
;;  AF, D
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
check_enemy_tile_collision::      
   ld     a, (hl)
   cp   #first_enemy_tile_pos
   ret    c
      ;; Hay colisión
      set  collision_enemy_bit, d
      ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Calcula la posición del tilemap donde cae el primer píxel
;; indicado según la posición X en C e Y en B
;; computa:
;;  HL = ( &tilemap + (y/8)*20 + x/4 )
;;                       fila    columna
;;     8 son los píxeles en h del tile 
;;     20 son los el número de tiles que caben en una fila
;;     4 son los píxeles por byte en modo 1
;; ENTRADAS
;;  C - Posición X
;;  B - Posición Y
;; SALIDAS
;;  HL - Puntero a la posicoón correcta del tilemap
;; MODIFICA
;;  BC, AF, DE, HL
;; PRECONDICIONES
;;  Los tiles deben ser de 32 bytes y el modo de video 1
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_collisions_getPtrInsideTilemapHL:: ;; ta chulo no?
   ld     h, #0
   ld     d, #0

   ;; b/=8
   srl    b
   srl    b
   srl    b          ;; división entera entre 8 => truncamos parte decimal => apuntamos al inicio del tile
   
   ;; HL = (b/8)*20
   sla    b
   sla    b          ;; x4
   ld     l, b       ;; l = (b/8)*4    ;;; ld a, b
   sla    b
   sla    b          ;; x16
   rl     h          ;; carry
   ld     e, b       
   add   hl, de      ;; HL = (b/8)*20  ;;; Se puede optimizar sumando parte baja y haciendo rl h

   ;; HL = (b/8)*20 + c/4
   srl    c                   
   srl    c          ;; división entera entre 4 => truncamos parte decimal = apuntamos al inicio del tile
   ld     e, c    
   add   hl, de      ;; HL = (b/8)*20 + c/4

   ;; HL = &tilemap + (y/8)*20 + x/4
   call man_game_getTilemapPtrDE
   add   hl, de
        
   ret
