;;	Copyright © CrackTime Studios - 2020
;;
;;	This file is part of Poisoned Escape
;;
;;    Poisoned Escape is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Poisoned Escape is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Poisoned Escape.  If not, see <https://www.gnu.org/licenses/>.

.module CONTROL_SYSTEM

.include "physics.h.s"
.include "cpct_functions.h.s"
.include "cpctelera.h.s"
.include "../man/entity.h.s"
.include "../man/game.h.s"
.include "render.h.s"
.include "../man/entity_templates.h.s"
.include "collisions.h.s"


;;======================================================================================
;; PUBLIC VARIABLES
;;======================================================================================


;;======================================================================================
;; PRIVATE VARIABLES
;;======================================================================================



;;======================================================================================
;; PUBLIC ROUTINES
;;======================================================================================

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Inicializa el sistema de physics
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  -
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_physics_init::
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Actualiza las físicas de todas las entidades que deban hacerlo
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  -
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_physics_update::
   ld    hl, #sys_physics_update_one_entity
   ld     c, #e_cmps_movable                ;; Todas las que tipo movibles
   call  man_entity_for_all_matching
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Aplica las físicas a una entidad
;; ENTRADAS
;;  IX - puntero a la entidad 
;; SALIDAS
;;  -
;; MODIFICA
;;  A
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_physics_update_one_entity::
   ld    a, e_cmps(ix)
   and  #e_cmps_gravitable
   jr    z, _limit_vy
      ld    a, e_vy(ix)
      cp    #max_vy
      jr    z, #_limit_vy
         ld    de, #gravity_acceleration    ;; ay ~= 0.5px/frame
         ld     h, e_vy(ix)
         ld     l, e_vyf(ix)
         add   hl, de
         ld    e_vy(ix),  h   
         ld    e_vyf(ix), l                        
  _limit_vy:
   ld     d, e_vy(ix)
   ld     e, e_vyf(ix)
   ld     h, e_y(ix)
   ld     l, e_yf(ix)
   add    hl, de
   ld    e_y(ix), h
   ld    e_yf(ix), l

   call sys_physics_round_XY

   ld    e_tile_collisions(ix), #0                 ;; Resetea las collisiones detectadas

   ld    a, e_cmps(ix)
   and   #e_cmps_tile_collisionable
   sub   #e_cmps_tile_collisionable
   jr    nz, #_tile_collisionable_Y_continue
      call sys_collisions_checkAndTreat_tile_collisionY
      call sys_physics_round_XY
   
  _tile_collisionable_Y_continue:
   ld     d, e_vx(ix)
   ld     e, e_vxf(ix)
   ld     h, e_x(ix)
   ld     l, e_xf(ix)
   add    hl, de
   ld    e_x(ix), h
   ld    e_xf(ix), l

   call sys_physics_round_XY                         

   ld    a, e_cmps(ix)
   and   #e_cmps_tile_collisionable
   sub   #e_cmps_tile_collisionable
   ret nz
      call  sys_collisions_checkAndTreat_tile_collisionX
      call  sys_physics_round_XY
      ld     a, e_tile_collisions(ix)
      and   #collision_enemy
      sub   #collision_enemy
      call   z, man_game_end_level

   ret

sys_physics_round_XY::
   ld     e, e_y(ix)
   ld     a, e_yf(ix)
   and   #0x80
   jr     z, round_y_continue 
      inc   e
   round_y_continue:
   ld    e_y_rounded(ix), e 

   ld     e, e_x(ix)
   ld     a, e_xf(ix)
   and   #0x80
   jr     z, round_x_continue 
      inc   e
   round_x_continue:
   ld    e_x_rounded(ix), e 
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Decide el comportamiento del player segúnlas colisiones
;; sufridas
;; 
;; ENTRADAS
;;  D  - Máscara de bordes colisionados
;;  IX - Puntero a entidad
;; SALIDAS
;;  -
;; MODIFICA
;;  AF
;; PRECONDICIONES
;;    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_physics_collision_behaviour::
   ld a, d
   and #collision_left
   sub #collision_left
   jr nz, _right
      call sys_physics_simulate_wall_friction
      
      ;; truncado parte fraccional
      ld     a, e_xf(ix)
      ld     b, e_vxf(ix)
      add    b
      jr    nc, carry_low_byte_left_continue
         inc   e_x(ix)
     carry_low_byte_left_continue:
      inc e_x(ix)
      ld e_xf(ix), #0
      ;;ld e_vxf(ix), #0

      jp _down
 _right:
   ld a, d
   and #collision_right
   sub #collision_right
   jr nz, _down
      call sys_physics_simulate_wall_friction

      ;; truncado parte fraccional
      ld    a, e_xf(ix)
      ld    b, e_vxf(ix)
      sub   b
      jr    nc, carry_low_byte_right_continue
         ld a, e_xf(ix)
         and #0x80
         jr nz, carry_low_byte_right_continue
            dec e_x(ix)
     carry_low_byte_right_continue:
      ld e_xf(ix), #0
      ;;ld e_vxf(ix), #0

  _down:
   ld   a, d
   and #collision_down
   sub #collision_down
   jr nz, _up
      ld     a, e_y(ix)
      cp    #0x80
      and   #0xFF-0b0111 
      ld    e_y(ix), a

      ld   e_vy(ix), #0
      ld   e_vyf(ix),#0
      ld   e_yf(ix), #0
      call man_game_floor_collision
      ret

  _up:
   ld a, d   
   and #collision_up
   sub #collision_up
   ret nz
      ld  a, e_y(ix)
      add   #8
      and   #0xFF-0b0111
      ld    e_y(ix), a
      ld    e_yf(ix), #0

      ld    h, e_vy(ix)
      ld    l, e_vyf(ix)
      ld    de, #gravity_acceleration
      add   hl, de
      ld    e_vy(ix), h
      ld    e_vyf(ix), l

      ret

sys_physics_simulate_wall_friction::
   ld     a, e_vy(ix)
   and     #0x80      
   jr     z, falling_friction
      ld     b, e_vy(ix) 
      ld     a, e_vyf(ix)
      sra    b
      rra
      ld    e_vy(ix),  b
      ld    e_vyf(ix), a
      ret
   falling_friction:   
   ld    a, e_tile_collisions(ix)
   and   #collision_down
   ret    nz
      ld     a, e_vy(ix)
      cp    #1
      ret    m
         ld    e_vy(ix), #1
         ld    e_vyf(ix), #0x00
ret
