;;	Copyright © CrackTime Studios - 2020
;;
;;	This file is part of Poisoned Escape
;;
;;    Poisoned Escape is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Poisoned Escape is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Poisoned Escape.  If not, see <https://www.gnu.org/licenses/>.

.include "render.h.s"
.include "cpctelera.h.s"
.include "cpct_functions.h.s"
.include "../man/entity.h.s"
.include "../man/game.h.s"
.include "../man/entity_templates.h.s"
.include "../assets/assets.h.s"
.include "../assets/maps/tilemap1.h.s"



;;======================================================================================
;; PUBLIC VARIABLES
;;======================================================================================

;;======================================================================================
;; PRIVATE VARIABLES
;;======================================================================================



;;======================================================================================
;; PUBLIC ROUTINES
;;======================================================================================

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Inicializa la paleta y modos de vídeo
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  TODO
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_render_init::
   halt
   ld    c, #1                    ;; Mode 0
   call cpct_setVideoMode_asm       ;; Set video mode
   ld   hl, #_main_palette           ;; Palette array
   ld   de, #4                     ;; Size of the palette
   call cpct_setPalette_asm         ;; Set the palette
   cpctm_setBorder_asm HW_BLACK    ;; Set the border (negro)      
   ;cpctm_setBorder_asm HW_WHITE     ;; Set the border (gris)      

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Renderiza todas las entidades renderizables
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  AF. DE, BC, HL, IX
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_render_update::
   ld    hl, #sys_render_one_entity
   ld     c, #e_cmps_renderable
   call  man_entity_for_all_matching
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Renderiza una entidad dada su dirección en IX
;; ENTRADAS
;;  IX - dirección de la entidad
;; SALIDAS
;;  -
;; MODIFICA
;;  DE, HL, BC, AF
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_render_one_entity::
   ;; Borra prevPtr si está inicializado
   ld     a, e_prevPtr_l(ix)
   sub    #0
   call   nz, sys_render_clean_prevPtr

   ld    a, e_type(ix)
   and   #e_type_dead
   ret   nz

   call prepare_registers_for_draw

   call  cpct_drawSprite_asm     ;; Modifica DE, HL, BC, AF
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Renderiza la muerte del jugador
;; ENTRADAS
;;  IX - dirección de la entidad
;; SALIDAS
;;  -
;; MODIFICA
;;  DE, HL, BC, AF
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_render_draw_death_animation::
   ld de, #0x07
   push de

   ; Borra prevPtr si está inicializado
   ld     a, e_prevPtr_l(ix)
   sub    #0
   call   nz, sys_render_clean_prevPtr

   ld    a, e_type(ix)
   and   #e_type_dead
   ret   nz
_continue:
   call prepare_registers_for_draw

   call  cpct_drawSprite_asm     ;; Modifica DE, HL, BC, AF
   call  cpct_waitVSYNC_asm
   ld d, #5
   call sys_render_wait
   pop de
   ld a, e
   sub #0x01
   or a
   jp z, _end_routine 

   ld e, a
   push de
   ld l, e_animPtr_l(ix)
   ld h, e_animPtr_h(ix)

   ld e, (hl)
   ld e_sprPtr_l(ix), e
   inc hl
   ld d, (hl)
   ld e_sprPtr_h(ix), d
   inc hl

   ld e_animPtr_l(ix), l                  ;; Guarda siguiente paso
   ld e_animPtr_h(ix), h                  ;; en la animacion

   jp _continue
_end_routine:
   ret

prepare_registers_for_draw:
   ;; Obtiene puntero a memoria de video
   ld    de, #memVideoPtr
   ld     c, e_x_rounded(ix)
   ld     b, e_y_rounded(ix) 
   call  cpct_getScreenPtr_asm   ;; Modifica DE, HL, BC, AF
   
   ;; HL = puntero a dirección de video
   ex    de, hl                  ;; DE: Puntero a dirección de video

   ld     l, e_sprPtr_l(ix)      ;; HL: sprite en memoria 
   ld     h, e_sprPtr_h(ix) 
   ld     c, e_w(ix)             ;; C: Ancho
   ld     b, e_h(ix)             ;; B: Alto

   ld    e_prevPtr_h(ix), e      ;; Actualiza prevPtr
   ld    e_prevPtr_l(ix), d
ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; pinta primer tilemap por ahora
;; ENTRADAS
;;  -
;; SALIDAS
;;  -
;; MODIFICA
;;  DE, HL, BC, AF, IX, IY
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_render_drawTilemap::
   ld    c, #20                  ;(1B C) width	Width in tiles of the view window to be drawn
   ld    b, #25                  ;(1B B) height	Height in tiles of the view window to be drawn
   ld    de, #20                 ;(2B DE) tilemapWidth	Width in tiles of the complete tilemap
   call man_game_getTilesetPtrHL ;(2B HL) tileset	Pointer to the start of the tileset definition (list of 32-byte tiles).
   call cpct_etm_setDrawTilemap4x8_ag_asm

   ld    hl, #memVideoPtr        ;(2B HL) memory	Video memory location where to draw the tilemap (character & 4-byte aligned)
   call man_game_getTilemapPtrDE ;(2B DE) tilemap	Pointer to the upper-left tile of the view to be drawn of the tilemap
   call cpct_etm_drawTilemap4x8_ag_asm
   ret

;;======================================================================================
;; PRIVATE ROUTINES
;;======================================================================================

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Borra con color de fondo la posición anterior
;; ENTRADAS
;;  IX - dirección de la entidad
;; SALIDAS
;;  -
;; MODIFICA
;;  DE, HL, BC, AF
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_render_clean_prevPtr::
   ld  e, e_prevPtr_h(ix)  ;(2B DE) memory	Video memory pointer to the upper left box corner byte
   ld  d, e_prevPtr_l(ix)
   ld  a, #backgroundColor ;(1B A ) colour_pattern	1-byte colour pattern (in screen pixel format) to fill the box with
   ld  c, e_w(ix)          ;(1B C ) width	Box width in bytes [1-64] (Beware!  not in pixels!)
   ld  b, e_h(ix)          ;(1B B ) height	Box height in bytes (>0)
   call cpct_drawSolidBox_asm
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Borra con color de fondo la posición anterior
;; ENTRADAS
;;  D - Número de frames
;; SALIDAS
;;  -
;; MODIFICA
;;  DE, BC, AF
;; PRECONDICIONES
;;  -  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_render_wait::
   halt
   halt
   call cpct_waitVSYNC_asm ;; Modifica AF, BC
   dec d
   jr nz, sys_render_wait
   ret

